//-------------------------------------------------------------------
// CX_PlugIn.h
//-------------------------------------------------------------------
// ACD SYSTEMS INTERNATIONAL INC.
// Copyright © 1994 - 2020 ACD Systems International Inc.
// All Rights Reserved
//
// NOTICE: ACD Systems International Inc. ("ACD") permits you to use,
// modify, and distribute this file in accordance with the terms of
// the ACDSee SDK license agreement accompanying it. If you have 
// received this file from a source other than ACD, then your use,
// modification, or distribution of it requires the prior written 
// permission of ACD.
//-------------------------------------------------------------------

/**********************************************************************************

Header containing datatypes and functions for ACD Command eXtension plug-ins.

All functions return a status code of 0 on success, or a negative error code
if a failure occurs, except for CX_ErrorHandler which must return one of the
CX_EHF_* flags.

The plug-in must export the following functions:

CX_Init              - Initialize the plug-in
CX_GetPlugInInfo     - Get plug-in information
CX_InvokeCommand     - Invoke a command implemented by the plug-in

The following functions are optional:

CX_ShowPlugInDialog  - Show plug-in about/configuration dialog
CX_Free              - Called immediately prior to the plug-in DLL being freed with FreeLibrary().
CX_Help              - Displays help for the plug-in or a specific command
CX_ErrorHandler      - The host calls this function if the plug-in causes an exception
CX_FormatInfoChanged - Checks if the format information for this plug-in has changed


**********************************************************************************/

#include <windows.h>

#ifndef _CX_PLUGIN_H_INCLUDED
#define _CX_PLUGIN_H_INCLUDED

#include "IE_PlugIn.h"

#ifndef PLUGIN_API
#define PLUGIN_API extern "C"
#endif

///////////////////
// MAKE_FORMATID //
///////////////////
#ifndef MAKE_FORMATID
#define MAKE_FORMATID(a,b,c,d) (((DWORD)(d)<<24) | ((DWORD)(c)<<16) | ((DWORD)(b)<<8) | ((DWORD)(a)))
#endif

/////////////////////
// Plug-in version //
/////////////////////
#define CX_VERSION 100

//////////////////
// Status codes //
//////////////////
enum
{
   CXE_OK                 =  0,  // Success
   CXE_Error              = -1,  // Unspecified error
   CXE_InvalidOp          = -2,  // Operation cannot be completed at this time
   CXE_InvalidParam       = -3,  // Invalid parameters
   CXE_NotImplemented     = -4,  // Missing implementation
   CXE_InvalidCommand     = -5,  // The specified command index is invalid
   CXE_Cancelled          = -6,  // Operation cancelled by user
   CXE_ImageDecodeFailure = -7,  // The specified image could not be decoded
   CXE_InvalidPage        = -8,  // The specified page index is invalid
   CXE_Malloc             = -9,  // Memory allocation error
   CXE_Decode             = -10, // Image decode error
   CXE_FatalError         = -11, // Unhandled exception
   CXE_HostVersion        = -12, // Host application does not support correct plug-in interface version
   CXE_NoMetadata         = -13, // The specified metadata item is not specified in the image
   CXE_FileNotFound       = -14, // The specified file could not be found
                                 // other errors...

   // Plug-in defined errors start at -1000
   CXE_FirstPlugInError   = -1000,
};

/////////////////
// CVF_* flags //
/////////////////
// Command validity flags
#define CVF_NOSINGLE     (1<< 0) // Command does not work with a single item
#define CVF_NOMULTIPLE   (1<< 1) // Command does not work with multiple items
#define CVF_NOFILES      (1<< 2) // Command does not work with files
#define CVF_NONONIMAGES  (1<< 3) // Command does not work with files other than images
#define CVF_NOFOLDERS    (1<< 4) // Command does not work with folders
#define CVF_NODRIVES     (1<< 5) // Command does not work with drives
#define CVF_NOREADONLY   (1<< 6) // Command does not work with read-only items
#define CVF_NOTEMPFILE   (1<< 7) // Command does not work with temporarily extracted files
#define CVF_FORMATIDLIST (1<< 8) // Command works only with image types specified by format ID array
#define CVF_CANENCODE    (1<< 9) // Command does not work with image formats that cannot be encoded
#define CVF_NIL          (1<<10) // Command works even if there are not selected items

///////////////////
// CX_CI_* flags //
///////////////////
// Command information flags
//
#define CX_CI_BTNLABEL     1   // The CX_CommandInfo::szBtnLabel member is valid
#define CX_CI_WANTLNKFILES 2   // The host should pass the raw paths of .lnk files (no resolution)
#define CX_CI_CATEGORIES   5   // The CX_CommandInfo::dwCategories and CX_CommandInfo::szBtnLabel members are valid

/////////////////
// IAF_* flags //
/////////////////
// Item attribute flags
enum
{
   IAF_FILE     = 4,   // File
   IAF_IMAGE    = 8,   // Image
   IAF_FOLDER   = 16,  // Folder
   IAF_DRIVE    = 32,  // Drive
   IAF_READONLY = 64,  // File cannot be modified
   IAF_TEMPFILE = 128, // File has been temporarily extracted (e.g., from an archive)
};

/////////////////
// CRF_* flags //
/////////////////
// Command result flags
enum
{
   CRF_FILECHANGE = 1, // One or more of the selected files has been changed
   CRF_NEWFILE    = 2, // One or more files were created 
};

/////////////////
// GTF_* flags //
/////////////////
// Get thumbnail flags
enum
{
   GTF_SIZE      = 1, // Use the requested thumbnail size
   GTF_DISPLAY   = 2, // Process the thumbnail for display
   GTF_COMMONPAL = 4, // Use common palette in 256 color mode
};

/////////////////
// DIF_* flags //
/////////////////
// Decode image flags
enum
{
   DIF_MAXSIZE   = 1, // Resize the image if necessary so it does not exceed the given maximum size
   DIF_DISPLAY   = 2, // Process the image for display
   DIF_COMMONPAL = 4, // Use common palette in 256 color mode
};

/////////////////
// CIF_* flags //
/////////////////
// Client info flags for CX_ClientInfo.
#define CIF_REGISTERED 1 // Calling application is the full version not the trial.
#define CIF_VERSION    2 // Version parameter is valid

//////////////////////////
// CX_MDT_* enumeration //
//////////////////////////
// Metadata data types
enum
{
   CX_MDT_STRING     = 1,   // Null-terminated ASCII string
   CX_MDT_STRINGLIST = 2,   // List of null-terminated ASCII strings terminated by a double-NULL
   CX_MDT_FILETIME   = 3,   // Date and time in FILETIME (UTC) format.
};

////////////////////
// CX_CPF_* flags //
////////////////////
// Flags for CX_CommandParams
#define CX_CPF_GETMETADATAITEM        (1<<0) // GetMetadataItem       callback function is valid
#define CX_CPF_GETFILEIMAGEINFO       (1<<1) // GetFileImageInfo      callback function is valid
#define CX_CPF_GETFILEPAGEINFO        (1<<2) // GetFilePageInfo       callback function is valid
#define CX_CPF_GETFILEIMAGETHUMBNAIL  (1<<3) // GetFileImageThumbnail callback function is valid
#define CX_CPF_DECODEFILEIMAGE        (1<<4) // DecodeFileImage       callback function is valid
#define CX_CPF_WRITEIMAGE             (1<<5) // WriteImage            callback function is valid
#define CX_CPF_ENUMENCODEFORMATS      (1<<7) // EnumEncodeFormats     callback function is valid
#define CX_CPF_ENCODEPARAMSDLG        (1<<6) // EncodeParamsDlg       callback function is valid
#define CX_CPF_GETFILEMETADATAITEM    (1<<8) // GetFileMetadataItem   callback function is valid
#define CX_CPF_SELECTITEMS            (1<<9) // SelectItems           callback function is valid
#define CX_CPF_SETFILEMETADATAITEM    (1<<10)// SetFileMetadataItem   callback function is valid
#define CX_CPF_GETORIGINALSFOLDER     (1<<13)// CX_GetOriginalsFolder callback function is valid
#define CX_CPF_GETDEVELOPEDFOLDER     (1<<14)// CX_GetDevelopedFolder callback function is valid
#define CX_CPF_CONVERTIMAGETYPETOJPG  (1<<15)// CX_ConvertImageTypeToJPG callback function is valid

//////////////
// CX_MDI_* //
//////////////
// Mapping of Metadata identifier to underlying data type.
#define CX_MDI_DESCRIPTION  "Description"  // The field is of type CX_MDT_STRING
#define CX_MDI_DATE         "Date"         // The field is of type CX_MDT_FILETIME
#define CX_MDI_AUTHOR       "Author"       // The field is of type CX_MDT_STRING
#define CX_MDI_NOTES        "Notes"        // The field is of type CX_MDT_STRING
#define CX_MDI_KEYWORDS     "Keywords"     // The field is of type CX_MDT_STRINGLIST
#define CX_MDI_RATING       "Rating"       // The field is of type CX_MDT_STRING
#define CX_MDI_RPP		    "RPP"			 // The field is of type CX_MDT_STRING

/////////////////////////////////////////////////
// CX_CN_* Command Extension Plugin categories //
/////////////////////////////////////////////////
#define CX_CN_ACQUIRE      (1<<0 ) // Tools to download or acquire media from external device or source
#define CX_CN_MANAGE       (1<<1 ) // Tools to manage media
#define CX_CN_EDIT         (1<<2 ) // Tools to edit media
#define CX_CN_PRINT        (1<<3 ) // Tools to print media
#define CX_CN_CREATE       (1<<4 ) // Tools to create documents from media
#define CX_CN_SHARE        (1<<5 ) // Tools to share media
#define CX_CN_UPLOAD       (1<<6 ) // Tools to share media by uploading to websites
#define CX_CN_RESERVED07   (1<<7 ) // Reserved for future category
#define CX_CN_RESERVED08   (1<<8 ) // Reserved for future category
#define CX_CN_RESERVED09   (1<<9 ) // Reserved for future category
#define CX_CN_RESERVED0A   (1<<10) // Reserved for future category
#define CX_CN_RESERVED0B   (1<<11) // Reserved for future category
#define CX_CN_RESERVED0C   (1<<12) // Reserved for future category
#define CX_CN_RESERVED0D   (1<<13) // Reserved for future category
#define CX_CN_RESERVED0E   (1<<14) // Reserved for future category
#define CX_CN_RESERVED0F   (1<<15) // Reserved for future category
#define CX_CN_CUSTOM0      (1<<16) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOM1      (1<<17) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOM2      (1<<18) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOM3      (1<<19) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOM4      (1<<20) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOM5      (1<<21) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOM6      (1<<22) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOM7      (1<<23) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOM8      (1<<24) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOM9      (1<<25) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOMA      (1<<26) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOMB      (1<<27) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOMC      (1<<28) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOMD      (1<<29) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOME      (1<<30) // Custom category (category defined by plug-in host)
#define CX_CN_CUSTOMF      (1<<31) // Custom category (category defined by plug-in host)

///////////////////////////
// CX_HC_* Help Commands //
///////////////////////////
#define CX_HC_PLUGIN  1 // Displays help for the plug-in.  
                        // dwData is not used.

#define CX_HC_COMMAND 2 // Displays help for a command.
                        // dwData is the index of the command.

////////////////////
// CX_EHF_* flags //
////////////////////
// Error Handler Flags
enum
{
   CX_EHF_DEFAULT      = 1, // Activate default host Error Handler 
   CX_EHF_DISABLE_PERM = 2, // Plug-in should be disabled by host app (permanently)
   CX_EHF_DISABLE_TEMP = 4, // Plug-in should be disabled by host app (temporarily)
};

//////////////
// CX_FSF_* //
//////////////
// Filter specification flags for CX_FitlerSpec
#define CX_FSF_NODRIVES      (1<<1) // Sequence cannot contain drive items
#define CX_FSF_NOFOLDERS     (1<<2) // Sequence cannot contain folder items
#define CX_FSF_NOFILES       (1<<3) // Sequence cannot contain file items
#define CX_FSF_ONLYIMAGES    (1<<4) // Sequence cannot contain files that are not images
#define CX_FSF_ONLYENCODABLE (1<<5) // Sequence cannot contain files that are not images in formats that can be encoded
#define CX_FSF_NOREADONLY    (1<<6) // Sequence cannot contain items with read only attribute
#define CX_FSF_NOTEMPITEMS   (1<<7) // Sequence cannot contain temporary items
#define CX_FSF_EXTENSIONLIST (1<<8) // Sequence cannot contain files with extensions other than those specified in pszExtList

//////////////////
// CX_ImageInfo //
//////////////////
// Image information
struct CX_ImageInfo
{
   CX_ImageInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD dwFlags;     // Flags -- set to 0
   SIZE  si;          // Image dimensions
   DWORD dwFormatID;  // Image format ID
   int   nBPP;        // Bits per pixel
   int   nBPS;        // Bits per sample
   int   nPages;      // Number of pages in image
};

/////////////////
// CX_PageInfo //
/////////////////
// Page information
struct CX_PageInfo
{
   CX_PageInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD dwFlags;     // Flags -- set to 0
   SIZE  si;          // Image dimensions
   int   nBPP;        // Bits per pixel
   int   nBPS;        // Bits per sample
};

/////////////////////
// CX_ProgressFunc //
/////////////////////
typedef int(__stdcall *CX_ProgressFunc)(void* pParam, int nProgressNum, int nProgressDen);

///////////////////////////
// CX_GetThumbnailParams //
///////////////////////////
// Thumbnail parameters and returned thumbnail
struct CX_GetThumbnailParams
{
   CX_GetThumbnailParams() { ZeroMemory(this, sizeof(*this)); };

   DWORD            dwFlags;    // GTF_* flags
   int              iPage;      // Page to get thumbnail for (0-based)
   SIZE             siThumb;    // Desired thumbnail dimensions
   HGLOBAL          hdib;       // Returned handle to thumbnail (must be freed by plug-in)
   CX_ProgressFunc  pfProgress; // Decoding progress callback function
   void*            pParam;     // Decoding progress callback parameter
};

//////////////////////////
// CX_DecodeImageParams //
//////////////////////////
// Image decode parameters and returned image
struct CX_DecodeImageParams
{
   CX_DecodeImageParams() { ZeroMemory(this, sizeof(*this)); };

   DWORD            dwFlags;    // DIF_* flags
   int              iPage;      // Page to decode (0-based)
   SIZE             siMax;      // Maximum image dimensions
   HGLOBAL          hdib;       // Handle to decoded image (must be freed by plug-in)
   CX_ProgressFunc  pfProgress; // Decoding progress callback function
   void*            pParam;     // Decoding progress callback parameter
};

///////////////////
// CX_ClientInfo //
///////////////////
struct CX_ClientInfo
{
   CX_ClientInfo() { ZeroMemory(this, sizeof(*this)); nVersion = CX_VERSION; };

   DWORD   dwFlags;       // CIF_* flags
   wchar_t szCompany[40]; // Name of company of calling application
   wchar_t szAppName[40]; // Name of calling application.
   int     nVersion;      // Interface version supported by client (valid only if CIF_VERSION specified)
};

////////////////////
// CX_CommandInfo //
////////////////////
struct CX_CommandInfo
{
   CX_CommandInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD    dwFlags;              // CX_CI_* Flags
   DWORD    dwValidFlags;         // CVF_* Validity flags
   DWORD    dwFormatID[16];       // Specifies acceptable image formats
   wchar_t  szName[40];           // Name of this command (used for toolbar button help hint)
   wchar_t  szMenuItemName[40];   // Menu item label for this command
   wchar_t  szDescription[80];    // Description of command (used for status bar help hint)
   UINT     iIcon;                // Command icon index (ordinal icon index in APL module resources)
   wchar_t  szBtnLabel[36];       // Button label (valid if CX_CI_BTNLABEL flag is set in dwFlags)
   DWORD    dwCategories;         // Command categories bitmask (valud if CX_CI_CATEGORIES flag is set in dwFlags)
};

///////////////////
// CX_PlugInInfo //
///////////////////
struct CX_PlugInInfo
{
   CX_PlugInInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD               dwFlags;       // Reserved - set to 0
   int                 nVersion;      // Plug-in specificiation version
   wchar_t             szTitle[40];   // Plug-in title
   UINT                iIcon;         // Plug-in icon index
   int                 nCommands;     // Number of commands supported
   CX_CommandInfo*     pCommandInfo;  // Information about each command supported
};

/////////////////
// CX_FileInfo //
/////////////////
struct CX_FileInfo
{
   CX_FileInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD               dwAttribs;     // IAF_* flags
   wchar_t*            pszFN;         // Full path of file
   DWORD               dwFormatID;    // Image file format ID (or 0 if not an image)
   DWORD               dwFlags;       // Flags (set to 0)
   void*               pParam;        // Application-specific data
};

//////////////////////
// CX_MetadataItem //
//////////////////////
// Metadata item parameter struct used by GetMetadataItem and SetMetadataItem callbacks.
// pszID is set to a Metadata item identifier (CX_MDI_*) by the plug-in.
// GetMetadataItem:
//    pData is allocated by the host application using HeapAlloc() on the process heap.
//    The plug-in is responsible for freeing pData using HeapFree(GetProcessHeap(), 0, pData).
// SetMetadataItem:
//    pData is managed by the plug-in.
struct CX_MetadataItem
{
   CX_MetadataItem() { ZeroMemory(this, sizeof(*this)); };

   DWORD               dwFlags;       // Flags (set to 0)
   char*               pszID;         // Item identifier (CX_MDI_*)
   void*               pData;         // Pointer to item value data
   UINT                cbData;        // Length of item value data
   UINT                uDataType;     // Type of item value data (CX_MDT_*)
};

///////////////////
// CX_FilterSpec //
///////////////////
// Filter specification used in CX_SelectItemsStruct.
// Plug-in is responsible for freeing pszExtList.
struct CX_FilterSpec
{
   CX_FilterSpec() { cbSize = sizeof(*this); dwFlags = 0; pszExtList = NULL; };

   UINT           cbSize;         // Size of this structure, in bytes
   DWORD          dwFlags;        // Combinations of CX_FSF_* flags
   const wchar_t* pszExtList;     // Double-NULL terminated list of extensions (e.g., "BMP\0DIB\0RLE\0")
};

//////////////////////////
// CX_SelectItemsStruct //
//////////////////////////
// Input/output parameter structure used by CX_SelectItems.
// hglSeqCur and hglSeqNew each reference a buffer containing a double null-terminated list of 
// fully-qualified file paths.  The plug-in is responsible for freeing hglSeqCur and hglSeqNew 
// using GlobalFree().
struct CX_SelectItemsStruct
{
   CX_SelectItemsStruct() {
      cbSize = sizeof(*this); dwFlags = 0; hwndParent = NULL; pFilter = NULL; hglSeqCur = NULL;
      pszDlgTitle = NULL; nMinSize = 0; nMaxSize = -1; *szLastFolder = L'\0'; hglSeqNew = NULL;
   };

   UINT             cbSize;                  // (in)  Size of this structure, in bytes
   DWORD            dwFlags;                 // (in)  Flags (set to 0)
   HWND             hwndParent;              // (in)  Parent window for dialog box.
   CX_FilterSpec*   pFilter;                 // (in)  Filter to enforce when selecting images, or NULL for no filter.
   HGLOBAL          hglSeqCur;               // (in)  Current image sequence.  Can be NULL.
   const wchar_t*   pszDlgTitle;             // (in)  Dialog box title, or NULL for default.
   int              nMinSize;                // (in)  Minimum number of items in the returned sequence
   int              nMaxSize;                // (in)  Maximum number of items in the returned sequence (or -1 for no limit)
   wchar_t          szLastFolder[MAX_PATH];  // (i/o) Folder last used for selecting candidate items
   HGLOBAL          hglSeqNew;               // (out) Modified image sequence.  Can be NULL.
};

///////////////////////////
// CX_ErrorHandlerParams //
///////////////////////////
// Error Handler Parameters
struct CX_ErrorHandlerParams
{
   CX_ErrorHandlerParams() { ZeroMemory(this, sizeof(*this)); };

   BOOL				fIsGUI;     // Does the host app have a GUI
};

////////////////////////////////////
// Application callback functions //
////////////////////////////////////
// GetImageInfo - Return image format and dimensions
typedef int(__stdcall *PF_CX_GetImageInfo) (
   CX_FileInfo*,
   CX_ImageInfo*);

// GetPageInfo - Return dimensions for a specific page of a multi-page image
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_GetPageInfo) (
   CX_FileInfo*,
   int iPage,
   CX_PageInfo*);

// GetThumbnail - Obtain image thumbnail
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_GetThumbnail) (
   CX_FileInfo*,
   CX_GetThumbnailParams*);

// DecodeImage - Decode image into a DIB
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_DecodeImage) (
   CX_FileInfo*,
   CX_DecodeImageParams*);

// GetMetadataItem - Get a specific piece of metadata associated with an image
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_GetMetadataItem) (
   CX_FileInfo*,
   int iPage,
   CX_MetadataItem*);

// GetMetadataItem - Get a specific piece of metadata associated with an image
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_GetFileMetadataItem) (
   const wchar_t* pszFN,
   int iPage,
   CX_MetadataItem*);

// GetFileImageInfo - Return image format and dimensions for a specified image file
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_GetFileImageInfo) (
   const wchar_t* pszFN,
   CX_ImageInfo*);

// GetFilePageInfo - Return dimensions for a specific page of a multi-page image file
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_GetFilePageInfo) (
   const wchar_t* pszFN,
   int iPage,
   CX_PageInfo*);

// GetFileImageThumbnail - Get a thumbnail for a specified image file
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_GetFileImageThumbnail) (
   const wchar_t* pszFN,
   CX_GetThumbnailParams*);

// DecodeFileImage - Decode a specified image file into a DIB
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_DecodeFileImage) (
   const wchar_t* pszFN,
   CX_DecodeImageParams*);

// Encode a DIB into an image file of the specified format and encoding parameters.
// Returns: an IEE_* result code (see IE_PlugIn.h)
typedef int(__stdcall *PF_CX_WriteImage) (
   BITMAPINFO*       pBMI,
   void*             pBits,
   DWORD             dwFormatId,
   IE_EncodeParams*  pep,
   const wchar_t*    pszOutputFN,
   int (__stdcall   *Callback)(void* pParam, int nNum, int nDen),
   void*             pParamCB);

// Returns the default encoding parameters for a specified format.
// Simply calls IEP_GetDefaultEncodeParams(), as defined in IE_PlugIn.h.
// Returns: an IEE_* result code (see IE_PlugIn.h)
typedef int(__stdcall *PF_CX_GetDefaultEncodeParams) (
   DWORD             dwFormatID,
   int*              pcbEncodeParams,
   IE_EncodeParams*  pEncodeParams);

// Show a dialog box allowing the user to adjust encoding parameters for the specified format.
// Returns: an IEE_* result code (see IE_PlugIn.h)
typedef int(__stdcall *PF_CX_EncodeParamsDlg) (
   HWND              hwndParent,
   DWORD             dwFormatId,
   IE_EncodeParams*  pep);

// Return an array of formats into which images may be encoded.
// Pass NULL for pfi to just get the number of formats.
// Returns: an IEE_* result code (see IE_PlugIn.h)
typedef int(__stdcall *PF_CX_EnumEncodeFormats) (
   int*              pnFormats,
   IE_FormatInfo**   pfi);

// Displays a modal dialog box in which the user can modify a sequence of items.
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_SelectItems) (
   CX_SelectItemsStruct* psi);

// SetMetadataItem - Set a specific piece of metadata associated with an image
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_SetFileMetadataItem) (
   const wchar_t* pszFN,
   int            iPage,
   CX_MetadataItem*);

// GetOriginalsFolder - Get the folder name of the originals folder
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_GetOriginalsFolder) (
   wchar_t*  pszOriginalsFolder, 
   const int nSize);

// GetDevelopedFolder - Get the folder name of the developed folder
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_GetDevelopedFolder) (
   wchar_t*  pszDevelopedFolder, 
   const int nSize);

// ConvertImageTypeToJPG - Convert raw,tiff image files to jpeg files
// Returns: a CXE_* result code.
typedef int(__stdcall *PF_CX_ConvertImageTypeToJPG) (
   const wchar_t*    pszSrcFile,
   const SIZE*       pTargetSize,
   IE_EncodeParams*  pEncodeParams,
   wchar_t*          pszDstFile,
   int(__stdcall    *Callback)(void* pParam, int nNum, int nDen),
   void*             pParam);

//////////////////
// CX_Callbacks //
//////////////////
// The size of this structure must be 100 bytes (25 pointers)
struct CX_Callbacks
{
   CX_Callbacks() { ZeroMemory(this, sizeof(*this)); };

   PF_CX_GetImageInfo           GetImageInfo;
   PF_CX_GetPageInfo            GetPageInfo;
   PF_CX_GetThumbnail           GetThumbnail;
   PF_CX_DecodeImage            DecodeImage;
   PF_CX_GetMetadataItem        GetMetadataItem;
   PF_CX_GetFileImageInfo       GetFileImageInfo;
   PF_CX_GetFilePageInfo        GetFilePageInfo;
   PF_CX_GetFileImageThumbnail  GetFileImageThumbnail;
   PF_CX_DecodeFileImage        DecodeFileImage;
   PF_CX_WriteImage             WriteImage;
   PF_CX_GetDefaultEncodeParams GetDefaultEncodeParams;
   PF_CX_EncodeParamsDlg        EncodeParamsDlg;
   PF_CX_EnumEncodeFormats      EnumEncodeFormats;
   PF_CX_GetFileMetadataItem    GetFileMetadataItem;
   PF_CX_SelectItems            SelectItems;
   PF_CX_SetFileMetadataItem    SetFileMetadataItem;
   void* reserved_a[2];
   PF_CX_GetOriginalsFolder     GetOriginalsFolder;
   PF_CX_GetDevelopedFolder     GetDevelopedFolder;
   PF_CX_ConvertImageTypeToJPG  ConvertImageTypeToJPG;
   void* reserved_b[4];
};

//////////////////////
// CX_CommandParams //
//////////////////////
struct CX_CommandParams
{
   CX_CommandParams() { ZeroMemory(this, sizeof(*this)); iCommand = -1; };

   DWORD            dwFlags;     // CX_CPF_* flags
   int              iCommand;    // Index of command to invoke
   int              nFiles;      // Number of files selected
   CX_FileInfo*     pfi;         // Array of information about selected files
   HWND             hwndParent;  // Host application parent window
   CX_Callbacks     cb;          // Callback function pointers
};

////////////////////////
// CX_CommandResults //
////////////////////////
struct CX_CommandResults
{
   CX_CommandResults() { ZeroMemory(this, sizeof(*this)); };

   DWORD dwFlags; // CRF_* flags
};

////////////////////////////////////////////////
// Function pointers for plug-in entry points //
////////////////////////////////////////////////
typedef int(__stdcall *PF_CX_Init)              (CX_ClientInfo*);
typedef int(__stdcall *PF_CX_Free)              ();
typedef int(__stdcall *PF_CX_GetPlugInInfo)     (CX_PlugInInfo**);
typedef int(__stdcall *PF_CX_ShowPlugInDialog)  (HWND);
typedef int(__stdcall *PF_CX_InvokeCommand)     (CX_CommandParams*, CX_CommandResults*);
typedef int(__stdcall *PF_CX_Help)              (HWND, UINT, DWORD);
typedef int(__stdcall *PF_CX_ErrorHandler)      (CX_ErrorHandlerParams*);
typedef int(__stdcall *PF_CX_FormatInfoChanged) (BOOL&);

//////////////////////////
// Plug-in entry points //
//////////////////////////

/**********
* CX_Init *
***********
* The host application will call this function once after loading the plug-in library. 
* The plug-in must not maintain the pointer to CX_ClientInfo; it should make a copy the information in the structure.
* The plug-in should store application-specific options in HKEY_CURRENT_USER\Software\<szCompany>\<szAppName> 
*/
PLUGIN_API int __stdcall CX_Init(
   CX_ClientInfo*  pci);  // (in) Information about the plug-in client (host)

/**********
* CX_Free *
***********
* The client application will call this function once when it no longer
* needs the plug-in.  After CX_Free() is called, no other entry points
* may be called.  Normally, CX_Free() will be called immediately before
* unloading the plug-in DLL using FreeLibrary().
*
* CX_Free() will not be called if CX_Init() failed.
*
* The main purpose of CX_Free() is to give the plug-in a chance to free
* any DLLs it has dynamically loaded with LoadLibrary().
*
* Returns: CXE_OK.
*/
PLUGIN_API int __stdcall CX_Free();

/*******************
* CX_GetPlugInInfo *
********************
* Returns a pointer to an CX_PlugInInfo.
* The plug-in owns the pointer and must not free it until the
* plug-in library is unloaded.
*/
PLUGIN_API int __stdcall CX_GetPlugInInfo(
   CX_PlugInInfo**  ppii);  // (out) Plug-in information

/*******************
* CX_InvokeCommand *
********************/
PLUGIN_API int __stdcall CX_InvokeCommand(
   CX_CommandParams*  pcp,  // (in ) Specifies command to invoke, and parameters
   CX_CommandResults* pcr); // (out) Command results

/**********************
* CX_ShowPlugInDialog *
***********************
* Shows an "About" and/or "Options" dialog for this plug-in.
*/
PLUGIN_API int __stdcall CX_ShowPlugInDialog(
   HWND        hwndParent); // (in ) Parent window

/**********
* CX_Help *
***********
* Displays help for the plug-in or a specific command.
*/
PLUGIN_API int __stdcall CX_Help(
   HWND        hWnd,        // Application window that is calling help                                 
   UINT        uCommand,    // Type of help requested (CX_HC_* value)
   DWORD       dwData);     // Additional data for help command

/******************
* CX_ErrorHandler *
*******************
* Handles an exception caused by the plug-in. The host will call this function if the
* plug-in causes an exception.
*
* The return value should be one of the CX_EHF_* flags.
*/
PLUGIN_API int __stdcall CX_ErrorHandler(
   CX_ErrorHandlerParams*    pehp);          // (in) Error Handler Parameters

/***********************
* CX_FormatInfoChanged *
************************
* Called by the plug-in host to determine whether or not to update any cached information about
* formats supported by this plug-in.
*
* Sets bChanged to TRUE if the host should update it's cache.
*/
PLUGIN_API int __stdcall CX_FormatInfoChanged(BOOL& bChanged);

#endif
