//-------------------------------------------------------------------
// ID_PlugIn.h
//-------------------------------------------------------------------
// ACD SYSTEMS INTERNATIONAL INC.
// Copyright © 1994 - 2020 ACD Systems International Inc.
// All Rights Reserved
//
// NOTICE: ACD Systems International Inc. ("ACD") permits you to use,
// modify, and distribute this file in accordance with the terms of
// the ACDSee SDK license agreement accompanying it. If you have 
// received this file from a source other than ACD, then your use,
// modification, or distribution of it requires the prior written 
// permission of ACD.
//-------------------------------------------------------------------

/**********************************************************************************
Header for for an ACD image decoding plug-in module.

The plug-in module must export the following functions:
ID_Init                 - Initializes the plug-in after it has been loaded
ID_GetPlugInInfo        - Returns information about the plug-in
ID_OpenImage            - Opens an image and initializes decoder state
ID_CloseImage           - Closes an image and frees decoder state
ID_GetImageInfo         - Returns information about an image
ID_GetPageInfo          - Returns information about a page in an image
ID_PageDecode           - Decodes a page of the image into a DIB

The following functions are optional:
ID_ShowPlugInDialog     - Displays the plug-in's "About box"
ID_GetError             - Retrieves a text description of a plug-in defined error code
ID_IsAutoRotated        - Automatically rotated based on EXIF
ID_GetRAWRotateInfo   - Gets rotate & flip values that need to be applied to the RAW file (not quite the same as getting exif orientation)
ID_PageDecodeStart      - Starts incremenal (pipelined) decoding
ID_PageDecodeStep       - Performs one step of incremental decoding
ID_PageDecodeStop       - Stops incremental decoding
ID_Free                 - Frees resources associate with the plug-in before it is unloaded
ID_Help                 - Requests help about the plug-in or a specific format
ID_CreateView           - Creates a plug-in defined view window that displays an image
ID_PreTranslateMessage  - Optional message filter for view windows
ID_ErrorHandler         - The host calls this function if the plug-in causes an exception
ID_FormatInfoChanged    - Checks if the format information for this plug-in has changed
ID_GetMetadataList      - Gets all the metadata of an image
ID_GetSupportedMetadata - Gets all supported types of metadata

The plug-in must allow multiple images to be decoded simultaneously.
However, for each image being decoded, only one page may be decoded
simultaneously.

**********************************************************************************/

#include <windows.h>

#ifndef _ID_PLUGIN_H_INCLUDED
#define _ID_PLUGIN_H_INCLUDED

#include "Metadata/Metadata.h"

#ifndef PLUGIN_API
#define PLUGIN_API extern "C"
#endif

///////////////////
// MAKE_FORMATID //
///////////////////
#ifndef MAKE_FORMATID
#define MAKE_FORMATID(a,b,c,d) (((DWORD)(d)<<24) | ((DWORD)(c)<<16) | ((DWORD)(b)<<8) | ((DWORD)(a)))
#endif

/////////////////////
// Plug-in version //
/////////////////////
#define ID_VERSION 200

//////////////////
// Status codes //
//////////////////
enum  // error codes:
{
   IDE_OK             =  0,  // success
   IDE_Error          = -1,  // unspecified error
   IDE_InvalidOp      = -2,  // operation cannot be completed at this time
   IDE_InvalidParam   = -3,  // invalid parameters
   IDE_NotImplemented = -4,  // missing implementation
   IDE_TruncatedData  = -5,  // premature EOF encountered
   IDE_UnknownFormat  = -6,  // unknown input format
   IDE_CorruptData    = -7,  // bad format / corrupt data
   IDE_TooBig         = -8,  // image is too big
   IDE_NoPage         = -9,  // specified page does not exist
   IDE_CreateBitmap   = -10, // error creating bitmap
   IDE_Cancelled      = -11, // operation cancelled by user
   IDE_NoMetadata     = -12, // no metadata exists for the specified page of the image
   IDE_NoAudio        = -13, // no audio clip exists for the image
   IDE_FatalError     = -14, // Unhandled exception
   IDE_EmbeddedData   = -15, // no embedded data exists for the specified page of the image
   IDE_NoColorProfile = -16, // no color profile exists for the image

   // other errors...

   // Plug-in defined errors start at -1000
   IDE_FirstPlugInError = -1000,
};

/////////////////
// CIF_* flags //
/////////////////
#define CIF_REGISTERED 1 // Calling application is registered

/////////////////
// IIF_* flags //
/////////////////
enum
{
   IIF_MULTIPAGE        = 1,  // Image has multiple pages
   IIF_ANIM             = 2,  // Image is an animation
   IIF_METADATATYPES    = 64, // Image has 1 or more metadata types and the metadata types variables are valid
   IIF_XTRANS           = 128 // This is a raw image with an XTrans sensor
};

/////////////////
// PPF_* flags //
/////////////////
// Note: The PPF_DPI and PPF_TRANSCOLOR flags are mutually exclusive
enum
{
   PPF_RGB           = 1,    // Image data is RGB
   PPF_GRAYSCALE     = 2,    // Image data is grayscale
   PPF_COLORMAP      = 4,    // Image data is colormapped
   PPF_ALPHA         = 8,    // Image has alpha channel
   PPF_VECTOR        = 16,   // Image is vector-based
   PPF_DPI           = 64,   // The siDPI field is valid
   PPF_TRANSCOLOR    = 512,  // This page has a transparent colour and the iTransColor field is valid
   PPF_METADATATYPES = 2048  // Page has 1 or more metadata types and the metadata types variables are valid
};

/////////////////
// PID_* flags //
/////////////////
enum
{
   PID_RECT                     = 1,   // Decode only the given rectanglular portion of the image
   PID_GET16BPC                 = 2,   // Decode 16-bits per channel (if available)
   PID_NEEDALPHACHANNEL         = 4,   // doesnt degrade alpha channeled images. ie 32 stays as 32 not returned 24bpp
   PID_DEMOSAIKED               = 8,	// Demosaik the RAW image using pixel grouping (will not modify original data) return default RPP(XML)
   PID_APPLY_DEVELOP_PROCESSING = 128, // Apply all develop processing to the RAW. Brushing is not supported at the moment 
};

/////////////////
// FIF_* flags //
/////////////////
enum
{
   FIF_THUMBNAILONLY = 1,  // Plug-in only generates a thumbnail representation of the file
   FIF_VIEWWINDOW    = 2,  // Plug-in can create a view window for displaying the image
   FIF_ISVIDEO       = 4,  // This "image" is actually a video file
   FIF_ISAUDIO       = 8,  // This "image" is actually an audio file
   FIF_HAS16BPC      = 16, // 16-bits per channel decode available
   FIF_ISRAW         = 32, // This "image" is actually a RAW file
   FIF_ISVECTOR      = 64, // This image is in vector format
   FIF_ISDOCUMENT    = 128,// This "image" is an office document
};


/////////////////
// IOF_* flags //
/////////////////
enum
{
   IOF_HASRAWPARAMEX   = 1,  // Has RAW ParamEx structure available
   IOF_ISTHUMBNAIL     = 2,  // Output image has been decoded from an embedded thumbnail image
   IOF_ISPREVIEW       = 4,  // Output image has been decoded from an embedded preview image
   IOF_ISROTATED       = 8,  // Output image has been rotated based on metadata (e.g. EXIF orientation)
   IOF_ISPREMULTIPLIED = 16, // Some formats require that the transparency value is applied to the pixel data
};

/////////////////
// POF_* flags //
/////////////////
enum
{
   POF_TOPDOWN     = 1,     // Output is by rows starting from the top
   POF_BOTTOMUP    = 2,     // Output is by rows starting from the bottom
   POF_LEFTTORIGHT = 4,     // Output is by columns starting from the left
   POF_RIGHTTOLEFT = 8,     // Output is by columns starting from the right
   POF_INTERLACED  = 16,    // Output is by random row
   POF_MULTIPASS   = 32,    // Output is done in multiple passes
   POF_TILED       = 64,    // Output is tiled
   POF_METAFILE    = 128,   // A metafile is generated
   POF_RECT        = 256,   // Only the given rectanglular portion of the image will be decoded
   POF_ISTHUMBNAIL = 512,   // Output image has been decoded from an embedded thumbnail image
   POF_ISPREVIEW   = 1024,  // Output image has been decoded from an embedded preview image
   POF_ISROTATED   = 2048,  // Output image has been rotated based on metadata (e.g. EXIF orientation)
};

///////////////////////
// PIQ_* enumeration //
///////////////////////
enum
{
   PIQ_USEEMBEDDEDTHUMBNAIL               = 0,  // Use an embedded thumbnail when available and applicable
   PIQ_SPEED                              = 1,  // Favour speed over quality (for display)
   PIQ_QUALITY                            = 2,  // Favour quality over speed (for editing/conversion)
   PIQ_USEEMBEDDEDTHUMBNAILALWAYSIFEXISTS = 3,  // Use the embedded thumbnail always if it exists
   PIQ_USEPREVIEWIMAGE                    = 4,  // Use the embedded preview image
   PIQ_DECODEFORRAWENCODE                 = 5,  // Used for encoding raw to DNG
   PIQ_USEFULLSIZEPREVIEWIMAGE            = 6   // Use the full size embedded preview image (it's usually faster in the viewer to decode the 
                                                // full size embedded JPG & avoid all the resampling)
};

/////////////////
// SIF_* flags //
/////////////////
enum
{
   SIF_VIRTUALFILENAME = 1, // The filename does not refer to an actual file
};

///////////////////////////
// ID_HC_* Help Commands //
///////////////////////////
#define ID_HC_PLUGIN   1 // Displays help for the plug-in.  
// dwData is not used.

#define ID_HC_FORMAT   2 // Displays help for an image format.  
// dwData is the index of the format.

////////////////////////////////
// ID_VWF_* View Window Flags //
////////////////////////////////
#define ID_VWF_PREVIEW 1 // Window is a preview rather than a full-size view

//////////////////////////
// ID_GAF_* Audio Flags //
//////////////////////////
#define ID_GAF_PAGENUMBER 1 // ID_Audio::iPage is specified

//////////////////////////////////
// ID_GCPF_* COLORPROFILE Flags //
//////////////////////////////////
#define ID_GCPF_PAGENUMBER 1 // ID_ColorProfile::iPage is specified. Reserved for future use.

////////////////////
// ID_BI_* Values //
////////////////////
// Use these flags instead of BI_RGB in BITMAPINFOHEADER::biCompression to pass
// back images using 16 bits per channel.
#define ID_BI_GRAY16 1000
#define ID_BI_RGB64  1001

///////////////////////////////////
// ID_VWM_* View Window Messages //
///////////////////////////////////

// ID_VWM_SETSOURCE (required)
// Specifies the image to be displayed.
// wReserved    = (WPARAM)wParam   // Reserved - set to 0
// lpszwFileName = (LPCSTR)lParam   // File containing image to display
// Returns: IDE_* result code.
// Note: the lpszwFileName buffer is temporary and must not be used after this
// message has been processed.
#define ID_VWM_SETSOURCE (WM_USER+5)

// ID_VWM_CALCMAGNIFICATION (required)
// Calculates the highest magnification level that can be fully displayed 
// within the given window dimensions.
// siSize = (SIZE*)wParam  // Pointer to a SIZE structure that specifies the window dimensions.  
// siMag  = (SIZE*)lParam  // Pointer to a SIZE structure that receives the magnification
//                         // level, where cx is set to the numerator, and cy is set to
//                         // the denominator.
// Returns: 1 on success, 0 on error.
#define ID_VWM_CALCMAGNIFICATION (WM_USER+2)

// ID_VWM_SETMAGNIFICATION (required)
// Specifies the magnification (zoom level) at which the image should be displayed.
// The magnification level is represented as a fractional number, where 1/1
// represents 100% magnification.
// wReserved  = (WPARAM)wParam  // Reserved - set to 0
// siMag      = (SIZE*)lParam   // Pointer to a SIZE structure that specifies the magnification
//                              // level, where cx is set to the numerator, and cy is set to  
//                              // the denominator.                                           
//
// Returns: 1 if magnification was set, 0 on error.
//
// Note: If the resulting display size of the image exceeds the window dimensions, then
// the view should be scrollable, either through the additional of scroll bars,
// or allowing the user to click and drag the image.

#define ID_VWM_SETMAGNIFICATION (WM_USER+3)

// ID_VWM_GETMAGNIFICATION (required)
// Retrieves the current magnification (zoom level).
// The magnification level is represented as a fractional number, where 1/1
// represents 100% magnification.
// pNumerator   = (UINT*)wParam // Address of UINT that receives the numerator
// pDenominator = (UINT)lParam  // Address of UINT that receives the denominator
// Returns: 1 on success, 0 on error.
#define ID_VWM_GETMAGNIFICATION (WM_USER+4)

////////////////////
// ID_EHF_* flags //
////////////////////
// Error Handler Flags
enum
{
   ID_EHF_DEFAULT      = 1, // Activate default host Error Handler 
   ID_EHF_DISABLE_PERM = 2, // Plug-in should be disabled by host app (permanently)
   ID_EHF_DISABLE_TEMP = 4, // Plug-in should be disabled by host app (temporarily)
};

/////////////////////
// ID_ProgressFunc //
/////////////////////
// Application-supplied progress callback function.
// Called by the plug-in periodically during image decoding.
// The decoding progress is indicated by (nProgressNum/nProgressDen), which 
// will vary over the decoding process starting with 0 and ending with 1 
// (nProgressNum==nProgressDen).
// Returns: The application returns a non-zero value to continue decoding.
// If zero is returned, then the plug-in should abort the decode process and
// return IDE_Cancelled.
// 
//  pParam           Application-defined value
//  nProgressNum     Numerator of progress
//  nProgressDen     Denominator of progress
typedef int(__stdcall *ID_ProgressFunc)(void* pParam, int nProgressNum, int nProgressDen);

///////////////////
// ID_ClientInfo //
///////////////////
struct ID_ClientInfo
{
   ID_ClientInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD    dwFlags;       // CIF_* flags
   wchar_t  szCompany[40]; // Name of company of calling application
   wchar_t  szAppName[40]; // Name of calling application.
};

///////////////////
// ID_FormatInfo //
///////////////////
// Specifies information about an image format supported bythe plug-in.
// The plug-in owns the memory pointed to by pszExtList and must not free it
// until the plug-in library is unloaded.
struct ID_FormatInfo
{
   ID_FormatInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD    dwFlags;        // FIF_* flags
   DWORD    dwID;           // unique identifier for this format
   wchar_t  szName[40];     // name of this format (e.g., "Windows BMP")
   wchar_t  szNameShort[8]; // short name of this format (e.g., "BMP")
   wchar_t* pszExtList;     // list of filename extensions for this format (e.g., "BMP\0DIB\0RLE\0")
   wchar_t  szDefExt[8];    // default extension for this format (e.g., "BMP")
   COLORREF color;          // background colour to use when highlighting
   UINT     iIcon;          // index of icon to display in Explorer
   wchar_t* pszMimeType;    // MIME type name
};

///////////////////
// ID_PlugInInfo //
///////////////////
// Specifies information about the plug-in.
// The plug-in owns the memory pointed to by pFormatInfo and must not free it until
// the plug-in library is unloaded.
struct ID_PlugInInfo
{
   ID_PlugInInfo() { ZeroMemory(this, sizeof(*this)); };
   DWORD          dwFlags;     // Flags -- set to 0
   int            nVersion;    // Plug-in specificiation version (100)
   wchar_t        szTitle[40]; // Plug-in title
   UINT           iIcon;       // Plug-in icon resource id
   int            nFormats;    // Number of formats supported by plug-in
   ID_FormatInfo* pFormatInfo; // Information about each format supported
};

//////////////////
// ID_ImageInfo //
//////////////////
// Information about an image.
struct ID_ImageInfo
{
   ID_ImageInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD    dwFlags;          // IIF_* flags
   DWORD    dwFormatID;       // format identifier
   SIZE     si;               // image dimensions
   int      nBPS;             // bits per sample
   int      nSPP;             // samples per pixel
   int      nPages;           // number of pages
   int      nMetadataTypes;   // Page has n Types of Metadata, valid if PPF_METADATATYPES
   HGLOBAL  hMetadataTypes;   // GUID array of metadata types on page, valid if PPF_METADATATYPES
};

///////////////////
// ID_SourceInfo //
///////////////////
// Defines the image source data.
// The archive plug-in must make a copy of the content of ID_SourceInfo.
// It may not retain a pointer to either ID_SourceInfo or its pszFN member
struct ID_SourceInfo
{
   ID_SourceInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD    dwFlags; // SIF_* flags
   wchar_t* pszFN;   // Name of image file, if applicable (can be NULL).
   BYTE*    pBuf;    // Pointer to image file data
   DWORD    dwLen;   // Byte length of image file
    
                     // Call-back function to read more data info buffer. 
                     // The function attempts to read as much additional data into buf as
                     // necessary to make dwBufRead at least as large as dwPos.
                     // Returns TRUE if successful, FALSE otherwise.
   BOOL(*pfFillBuffer)(void* pUserParam, DWORD dwPos, DWORD* pdwNewPos);
   void* pParam;     // Calling application can set this to whatever it likes
};

//////////////////
// IDP_PageInfo //
//////////////////
// Information about a page of an image.
struct ID_PageInfo
{
   ID_PageInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD    dwFlags;          // PPF_* flags
   SIZE     si;               // Page dimensions
   int      nBPS;             // Bits per sample
   int      nSPP;             // Samples per pixel
   int      nDelay;           // Frame delay in ms (for animation)
   wchar_t  szTitle[32];      // Page title 

   union
   {
      SIZE  siDPI;            // Image resolution in DPI (valid only if the PPF_DPI flag is given)
      int   iTransColor;      // Transparent colour index (valid only if PPF_TRANSCOLOR flag is given)
   };
   int      nMetadataTypes;   // Page has n Types of Metadata, valid if PPF_METADATATYPES
   HGLOBAL  hMetadataTypes;   // GUID array of metadata types on page, valid if PPF_METADATATYPES
};

/////////////////
// ID_ImageOut //
/////////////////
// Defines the output of the ID_PageDecode function.
// The calling application is responsible for freeing hdib and hemf
// using GlobalFree() and DeleteEnhMetaFile(), resp.
struct ID_ImageOut
{
   ID_ImageOut() { ZeroMemory(this, sizeof(*this)); };

   DWORD        dwFlags;	// IOF_* flags
   HGLOBAL      hdib;		// handle to DIB
   HENHMETAFILE hemf;		// handle to an enhanced metafile
   RECT         rc;			// Rectangle defining area of source image decoded
   void*		pParamEx;	   // Extended parameter
};

////////////////////
// ID_DecodeParam //
////////////////////
struct ID_DecodeParam
{
   ID_DecodeParam() { ZeroMemory(this, sizeof(*this)); quality = PIQ_SPEED; };

   DWORD            dwFlags;		// PID_* flags
   int              nPage;			// Page to decode
   RECT             rc;				// Region of page to decode, in source pixels (if PID_RECT specified)
   int              nWidth;	   // Desired width  of output image/region
   int              nHeight;     // Desired height of output image/region
   int              quality;		// PIQ_* value
   ID_ProgressFunc  pf;				// Progress callback function
   void*            pParam;		// Progress callback parameter
   void*			     pParamEx;		// Extended parameter
};

////////////////////
// ID_OutputParam //
////////////////////
struct ID_OutputParam
{
   ID_OutputParam() { ZeroMemory(this, sizeof(*this)); };

   DWORD			   dwFlags;        // POF_* flags
   RECT				rc;             // Region of source page to be decoded, in source pixels (if POF_RECT specified)
   int				nWidth;         // Output width
   int				nHeight;        // Output height
   int				bpp;            // Output bits per pixel (1, 4, 8, 16 or 24)
   int				nColorMapLen;   // #Entries in output colormap
   RGBQUAD			colormap[256];  // Output colormap   
};

///////////////////
// ID_StepOutput //
///////////////////
struct ID_StepOutput
{
   ID_StepOutput() { ZeroMemory(this, sizeof(*this)); };

   DWORD dwFlags;          // Flags -- set to 0
   RECT  rc;               // Rectangle defining area of image decoded (in dest. pixels)
   BYTE* pBuf;             // Pointer to buffer containing output data.  (The Plug-in owns the buffer.  The calling application will use the buffer right away and will not modify its content.)
   int   nProgressNumPage; // Decoding progress of current page numerator
   int   nProgressDenPage; // Decoding progress of current page denominator
   int   nProgressNumAll;  // Decoding progress of entire image (all pages) numerator  
   int   nProgressDenAll;  // Decoding progress of entire image (all pages) denominator
};

//////////////
// ID_Audio //
//////////////
// Caller sets pszAudioFN to the name of a file in which to write the audio.
// Plug-in extracts embedded audio into this file.
struct ID_Audio
{
   ID_Audio() { ZeroMemory(this, sizeof(*this)); };

   DWORD    dwFlags;         // (in) ID_GAF_* flags
   wchar_t* pszAudioFN;      // (in) Path to audio file
   int      iPage;           // (in) Specifies page of image for which sound is to be retrieved 
                             // (valid only if ID_GAF_PAGENUMBER is specified)
};

/////////////////////
// ID_COLORPROFILE //
/////////////////////
// Caller sets pszColorProfileFN to the name of a file in which to write the colorprofile.
// Plug-in extracts embedded profile into this file.
struct ID_ColorProfile
{
   ID_ColorProfile() { ZeroMemory(this, sizeof(*this)); };

   DWORD    dwFlags;             // (in) ID_GCPF_* flags
   wchar_t* pszColorProfileFN;   // (in) Path to file
   int      iPage;               // (in) [Future Use] (valid only if ID_GCPF_PAGENUMBER is specified)
};

////////////////////////
// ID_CreateViewParam //
////////////////////////
// Used with ID_CreateView() function.
struct ID_CreateViewParam
{
   ID_CreateViewParam() { ZeroMemory(this, sizeof(*this)); };

   DWORD dwFlags;         // (in)  ID_VWF_* Flags
   DWORD dwFormatID;      // (in)  Format ID of image that will be displayed in view window
   HWND  hwndParent;      // (in)  Handle to parent window
   HWND  hwndView;        // (out) Receives handle to create view window
};

///////////////////////////
// ID_ErrorHandlerParams //
///////////////////////////
// Error Handler Parameters
struct ID_ErrorHandlerParams
{
   ID_ErrorHandlerParams() { ZeroMemory(this, sizeof(*this)); };

   BOOL				fIsGUI;     // Does the host app have a GUI
};

/////////////////
// ID_StateHdl //
/////////////////
typedef void* ID_StateHdl;

///////////////////////
// Function pointers //
///////////////////////
typedef int(__stdcall *PF_ID_Init)                 (ID_ClientInfo*);
typedef int(__stdcall *PF_ID_Free)                 ();
typedef int(__stdcall *PF_ID_GetPlugInInfo)        (ID_PlugInInfo**);
typedef int(__stdcall *PF_ID_ShowPlugInDialog)     (HWND);
typedef int(__stdcall *PF_ID_GetError)             (int, wchar_t*, int);
typedef int(__stdcall *PF_ID_OpenImage)            (ID_SourceInfo*, ID_StateHdl*);
typedef int(__stdcall *PF_ID_CloseImage)           (ID_StateHdl);
typedef int(__stdcall *PF_ID_GetImageInfo)         (ID_StateHdl, ID_ImageInfo*);
typedef int(__stdcall *PF_ID_GetPageInfo)          (ID_StateHdl, int nPage, ID_PageInfo*);
typedef int(__stdcall *PF_ID_IsAutoRotated)        (ID_StateHdl);
typedef int(__stdcall *PF_ID_GetRAWRotateInfo)     (ID_StateHdl, int& iRotateAngle, BOOL& bVerticalFlip);
typedef int(__stdcall *PF_ID_PageDecode)           (ID_StateHdl, ID_DecodeParam*, ID_ImageOut*);
typedef int(__stdcall *PF_ID_PageDecodeStart)      (ID_StateHdl, ID_DecodeParam*, ID_OutputParam*);
typedef int(__stdcall *PF_ID_PageDecodeStep)       (ID_StateHdl, ID_StepOutput*);
typedef int(__stdcall *PF_ID_PageDecodeStop)       (ID_StateHdl);
typedef int(__stdcall *PF_ID_Help)                 (HWND, UINT, DWORD);
typedef int(__stdcall *PF_ID_CreateView)           (ID_CreateViewParam*);
typedef int(__stdcall *PF_ID_PreTranslateMessage)  (HWND, MSG*);
typedef int(__stdcall *PF_ID_ErrorHandler)         (ID_ErrorHandlerParams*);
typedef int(__stdcall *PF_ID_FormatInfoChanged)    (BOOL&);
typedef int(__stdcall *PF_ID_GetMetadataList)      (ID_StateHdl, int, int, GUID*, MD_MetadataList*);
typedef int(__stdcall *PF_ID_GetSupportedMetadata) (DWORD dwFormatID, int&, GUID**);

/**********
* ID_Init *
***********
* The client application will call this function once after loading the
* plug-in library.  The plug-in must not maintain the pointer to
* ID_CallerInfo; it should make a copy the information in the structure.
* The plug-in should store application-specific options in
* HKEY_CURRENT_USER\Software\<szCompany>\<szAppName>
*/
PLUGIN_API int __stdcall ID_Init(
   ID_ClientInfo* pci); // (in) Information about the plug-in client (host)

/**********
* ID_Free *
***********
* The client application will call this function once when it no longer
* needs the plug-in.  After ID_Free() is called, no other entry points
* may be called.  Normally, ID_Free() will be called immediately before
* unloading the plug-in DLL using FreeLibrary().
*
* ID_Free() will not be called if ID_Init() failed.
*
* The main purpose of ID_Free() is to give the plug-in a chance to free
* any DLLs it has dynamically loaded with LoadLibrary().
*
* Returns: IDE_OK.
*/
PLUGIN_API int __stdcall ID_Free();

/*******************
* ID_GetPlugInInfo *
********************
* Returns a pointer to an ID_PlugInInfo struct.
* The plug-in owns the pointer and must not free it until the
* plug-in library is unloaded.
*/
PLUGIN_API int __stdcall ID_GetPlugInInfo(
   ID_PlugInInfo** pi); // (out) Plug-in information


/**********************
* ID_ShowPlugInDialog *
***********************
* Show an "About" and/or "Options" dialog for this plug-in.
*/
PLUGIN_API int __stdcall ID_ShowPlugInDialog(
   HWND  hwndParent); // Parent window

/**************
* ID_GetError *
***************
* Load pszErrBuf with a null-terminated string describing the given
* plug-in specific error code.
*/
PLUGIN_API int __stdcall ID_GetError(
   int      nError,      // error code
   wchar_t* pszErrBuf,   // buffer to put error message text info
   int      cbErrBuf);   // size of error buffer

/***************
* ID_OpenImage *
****************/
PLUGIN_API int __stdcall ID_OpenImage(
   ID_SourceInfo* psi,   // [in]  Image source data.
   ID_StateHdl*   phs);  // [out] Image state handle

/****************
* ID_CloseImage *
*****************/
PLUGIN_API int __stdcall ID_CloseImage(
   ID_StateHdl hs);      // [in]  Image state handle

/******************
* ID_GetImageInfo *
*******************/
PLUGIN_API int __stdcall ID_GetImageInfo(
   ID_StateHdl   hs,     // [in]  Image state handle
   ID_ImageInfo* pii);   // [out] Image information

/*****************
* ID_GetPageInfo *
******************/
PLUGIN_API int __stdcall ID_GetPageInfo(
   ID_StateHdl   hs,      // [in]  Image state handle
   int           iPage,   // [in]  Page number (0-based) to get information for
   ID_PageInfo*  ppi);    // [out] Page information

/*******************
* ID_IsAutoRotated *
********************
* return IDE_OK if it is auto rotated
* return IDE_Error if it is not.
*/
PLUGIN_API int __stdcall ID_IsAutoRotated(
   ID_StateHdl     hs);   // [in]  Image state handle

/**********************
* ID_GetRAWRotateInfo *
***********************/
PLUGIN_API int __stdcall ID_GetRAWRotateInfo(
   ID_StateHdl     hs,     // [in]  Image state handle
   int& iRotateAngle,      // [out] Angle image needs to be rotated by
   BOOL& bVerticalFlip);   // [out] Does image need to be vertically flipped

/****************
* ID_PageDecode *
*****************/
PLUGIN_API int __stdcall ID_PageDecode(
   ID_StateHdl     hs,   // [in]  Image state handle
   ID_DecodeParam* pdp,  // [in]  Decoding parameters
   ID_ImageOut*    pio); // [out] Output parameters  

/**********************
* ID_PageDecodeStart *
**********************/
PLUGIN_API int __stdcall ID_PageDecodeStart(
   ID_StateHdl     hs,   // [in]  Image state handle
   ID_DecodeParam* pdp,  // [in]  Decoding parameters
   ID_OutputParam* pop); // [out] Output parameters

/********************
* ID_PageDecodeStep *
*********************/
PLUGIN_API int __stdcall ID_PageDecodeStep(
   ID_StateHdl    hs,    // [in]  Image state handle
   ID_StepOutput* pso);  // [out] Output for this step

/********************
* ID_PageDecodeStop *
*********************/
PLUGIN_API int __stdcall ID_PageDecodeStop(
   ID_StateHdl    hs);   // [in]  Image state handle

/************
* ID_Help *
************
* Displays help for a plug-in or image format.
*/
PLUGIN_API int __stdcall ID_Help(
   HWND        hWnd,       // Application window that is calling help                                 
   UINT        uCommand,   // Type of help requested (ID_HC_* value)
   DWORD       dwData);    // Additional data for help command

/******************
* ID_CreateView *
******************
* Creates a plug-in controlled window containing a view of an image file.
* The window should be created as a child window and should be initially
* non-visible (WS_VISIBLE flag unset).  The child window should not
* use any borders (i.e., do not use window styles such as WS_BORDER
* or WS_EX_CLIENTEDGE).
* Once created, the plug-in host application can perform the following
* operations on the window:
*      Change window size and position
*      Send ID_VWM_* messages to the window
*      Destroy the window
* The FIF_VIEWINDOW flag must be included in ID_FormatInfo::dwFlags for
* those image formats where a View Window is supported by the plug-in.
*/
PLUGIN_API int __stdcall ID_CreateView(
   ID_CreateViewParam* pCVS); // View creation parameters

/***************************
* ID_PreTranslateMessage *
***************************
* Returns TRUE if the plug-in has handled the message, FALSE if the application should handle it.
*/
PLUGIN_API int __stdcall ID_PreTranslateMessage(
   HWND        hwndView,    // (in)  Handle of the plug-in's view window
   MSG*        pMsg);       // (in ) Message struct

/********************
* ID_ErrorHandler *
********************
* Handles an exception caused by the plug-in. The host will call this function if the
* plug-in causes an exception.
*
* The return value should be one of the ID_EHF_* flags.
*/
PLUGIN_API int __stdcall ID_ErrorHandler(
   ID_ErrorHandlerParams*    pehp);          // (in) Error Handler Parameters

/*************************
* ID_FormatInfoChanged *
*************************
* Called by the plug-in host to determine whether or not to update any cached information about
* formats supported by this plug-in.
*
* Sets bChanged to TRUE if the host should update it's cache.
*/
PLUGIN_API int __stdcall ID_FormatInfoChanged(BOOL& bChanged);

/***********************
* ID_GetMetadataList *
***********************
*
* Gets all or some of the metadata in an image. nMetaTypes = 0 and guidTypes == NULL to get all.
*/
PLUGIN_API int __stdcall ID_GetMetadataList(
   ID_StateHdl hs, 
   int iPage, 
   int nMetaTypes, 
   GUID* guidTypes, 
   MD_MetadataList* pList
);

/***********************
* GetSupportedMetadata *
************************
*
* Gets the list of supported metadata types for the image format specified.
*/
PLUGIN_API int __stdcall ID_GetSupportedMetadata(
   DWORD dwFormatID,                   // (in ) Format ID
   int& nFormats,                      // (out) Number of supported formats
   GUID** pFormats);                   // (out) GUIDs of supported formats


#endif
