//-------------------------------------------------------------------
// IE_PlugIn.h
//-------------------------------------------------------------------
// ACD SYSTEMS INTERNATIONAL INC.
// Copyright © 1994 - 2020 ACD Systems International Inc.
// All Rights Reserved
//
// NOTICE: ACD Systems International Inc. ("ACD") permits you to use,
// modify, and distribute this file in accordance with the terms of
// the ACDSee SDK license agreement accompanying it. If you have 
// received this file from a source other than ACD, then your use,
// modification, or distribution of it requires the prior written 
// permission of ACD.
//-------------------------------------------------------------------

/**********************************************************************************

Header containing datatypes and functions for ACD Image Export plug-in modules.

All functions return a status code of 0 on success, or a negative error code
if a failure occurs, except for IE_ErrorHandler which must return one of the
IE_EHF_* flags. Status codes are defined in APlugIn.h and IPlugIn.h.

Required functions:
IE_Init
IE_GetPlugInInfo
IE_ShowPlugInDialog
IE_GetValidPixelFormats
IE_ImageEncode

Optional functions:
IE_GetError
IE_GetDefaultEncodeParams
IE_ShowEncodeParamsDialog
IE_ImageEncodeStart
IE_ImageEncodePage
IE_ImageEncodeFinish
IE_ImageEncodeToBuffer
IE_Free
IE_Help
IE_ErrorHandler
IE_FormatInfoChanged
IE_SetMetadataList
IE_GetSupportedMetadata
IE_GetNativelySupportedMetadata
IE_SetEncodingInfo

**********************************************************************************/

#ifndef _IE_PLUGIN_H_INCLUDED
#define _IE_PLUGIN_H_INCLUDED

#include <windows.h>
#include "Metadata/Metadata.h"

#ifndef PLUGIN_API
#define PLUGIN_API extern "C"
#endif

///////////////////
// MAKE_FORMATID //
///////////////////
#ifndef MAKE_FORMATID
#define MAKE_FORMATID(a,b,c,d) (((DWORD)(d)<<24) | ((DWORD)(c)<<16) | ((DWORD)(b)<<8) | ((DWORD)(a)))
#endif

/////////////////////
// Plug-in version //
/////////////////////
#define IE_VERSION 100

//////////////////
// Status codes //
//////////////////
enum  // error codes:
{
   IEE_OK               =  0,   // Success
   IEE_Error            = -1,   // Unspecified error
   IEE_InvalidOp        = -2,   // Operation cannot be completed at this time
   IEE_InvalidParam     = -3,   // Invalid parameters
   IEE_NotImplemented   = -4,   // Missing implementation
   IEE_WriteFailure     = -5,   // File write failed
   IEE_TooBig           = -6,   // Image is too big
   IEE_Cancelled        = -7,   // Operation cancelled by user
   IEE_InvalidPixFormat = -8,   // The given pixel format is not supported
   IEE_MultiPage        = -9,   // Multiple pages not supported
   IEE_UnknownFormat    = -10,  // The specified format ID is not known
   IEE_StructSize       = -11,  // The specified structure size is too small
   IEE_FatalError       = -12,  // Unhandled exception
   IEE_BadAudio         = -13,  // The specified audio file is invalid or does not exist
   IEE_BadImage         = -14,  // The specified image file is invalid or does not exist
   IEE_BadColorProfile  = -15,  // The specified color profile is invalid
                                // other errors...

   // Plug-in defined errors start at -1000
   IEE_FirstPlugInError = -1000,
};

/////////////////
// CIF_* flags //
/////////////////
#define CIF_REGISTERED 1 // Calling application is registered

/////////////////
// FIF_* flags //
/////////////////
// Format information flags, used in IE_FormatInfo
//
enum
{
   FIF_ENCODESETTINGS                     = 1,    // Encode settings supported
   FIF_MULTIPAGE                          = 2,    // Multi-page operation supported
   FIF_ANIM                               = 4,    // Animations supported
   FIF_EDITMODE                           = 8,    // Encoder can open an existing image in edit mode
   FIF_ENCODER_REQUIRES_RAW_DECODING_INFO = 1024,
   FIF_LAYERS                             = 2048, //Layered image supported
   FIF_REQUIRES_PREMULTIPLIED_DATA        = 4096, //Premultiplied image data required when encoding this format

};

/////////////////
// EPF_* flags //
/////////////////
enum
{
   EPF_SAVESETTINGS = 1,    // Specifies that the settings should be saved as default values
};

////////////////////////////////
// PFF_* - Pixel format flags //
////////////////////////////////
// Formats designated by a '*' in the comment are new pixel formats for ACDSee 7.
// See IE_GetValidPixelFormats()
#ifndef PFF_FLAGS
#define PFF_FLAGS
enum
{
   // Exactly one of these flags must be specified
   PFF_1BPS    = 1,  //  1 bit  per sample (colormapped or grayscale only)
   PFF_4BPS    = 4,  //  4 bits per sample (colormapped or grayscale only)
   PFF_5BPS    = 5,  //  5 bits per sample (RGB only)
   PFF_8BPS    = 8,  //  8 bits per sample
   PFF_16BPS   = 16, //  16 bits per sample*
   PFF_BPSMASK = 31, //  Mask for bits per sample

   // Exactly one of these flags must be specified
   PFF_RGB   = 32,  // RGB color
   PFF_GRAY  = 64,  // Grayscale
   PFF_CMAP  = 128, // Colormapped
   PFF_ALPHA = 256, // Alpha Channel

   PFF_1BPP_GRAY  = PFF_1BPS      | PFF_GRAY,  //  1 bpp grayscale
   PFF_1BPP_CMAP  = PFF_1BPS      | PFF_CMAP,  //  1 bpp colormapped
   PFF_4BPP_GRAY  = PFF_4BPS      | PFF_GRAY,  //  4 bpp grayscale  
   PFF_4BPP_CMAP  = PFF_4BPS      | PFF_CMAP,  //  4 bpp colormapped
   PFF_8BPP_GRAY  = PFF_8BPS      | PFF_GRAY,  //  8 bpp grayscale  
   PFF_8BPP_CMAP  = PFF_8BPS      | PFF_CMAP,  //  8 bpp colormapped
   PFF_15BPP_RGB  = PFF_5BPS      | PFF_RGB,   // 15 bpp RGB colour
   PFF_24BPP_RGB  = PFF_8BPS      | PFF_RGB,   // 24 bpp RGB colour
   PFF_32BPP_RGB  = PFF_24BPP_RGB | PFF_ALPHA, // 32 bpp RGB Alpha channel [RGBQUAD]
   PFF_64BPP_RGB  = PFF_16BPS     | PFF_RGB,   // 64 bpp RGB colour *
   PFF_16BPP_GRAY = PFF_16BPS     | PFF_GRAY,  // 16 bpp grayscale *
};
#endif

//////////////
// IE_PIF_* // 
//////////////
// Flags used in the IE_PageInfo structure.
#define IE_PIF_METADATALIST        8 // The metadata list member is valid

////////////////////
// IE_BI_* Values //
////////////////////
// These flags are used instead of BI_RGB in BITMAPINFOHEADER::biCompression for
// images using 16 bits per channel.
#define IE_BI_GRAY16 1000
#define IE_BI_RGB64  1001

///////////////////////////
// IE_HC_* Help Commands //
///////////////////////////
#define IE_HC_PLUGIN 1 // Displays help for the plug-in.  
// dwData is not used.

#define IE_HC_FORMAT 2 // Displays help for a format.
// dwData is the index of the format.

////////////////////
// IE_EHF_* flags //
////////////////////
// Error Handler Flags
enum
{
   IE_EHF_DEFAULT      = 1, // Activate default host Error Handler 
   IE_EHF_DISABLE_PERM = 2, // Plug-in should be disabled by host app (permanently)
   IE_EHF_DISABLE_TEMP = 4, // Plug-in should be disabled by host app (temporarily)
};

///////////////////
// IE_ClientInfo //
///////////////////
struct IE_ClientInfo
{
   IE_ClientInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD   dwFlags;       // CIF_* flags
   wchar_t szCompany[40]; // Name of company of calling application
   wchar_t szAppName[40]; // Name of calling application.
};

///////////////////
// IE_FormatInfo //
///////////////////
struct IE_FormatInfo
{
   IE_FormatInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD    dwFlags;              // FIF_* Flags
   DWORD    dwID;                 // Unique identifier for this format
   wchar_t  szName[40];           // Name of this format (e.g., "Windows BMP")
   wchar_t  szNameShort[8];       // Short name of this format (e.g., "BMP")
   wchar_t  szDefExt[8];          // Default extension for this format (e.g., "BMP")
   UINT     iIcon;                // Format icon resource identifier
};


///////////////////
// IE_PlugInInfo //
///////////////////
struct IE_PlugInInfo
{
   IE_PlugInInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD            dwFlags;       // Flags (set to 0)
   int              nVersion;      // Plug-in specificiation version
   wchar_t          szTitle[40];   // Plug-in title
   UINT             iIcon;         // Plug-in icon resource id
   int              nFormats;      // Number of formats exported to
   IE_FormatInfo*   pFormatInfo;   // Information about each format supported
};

//////////////////
// IE_ImageInfo //
//////////////////
struct IE_ImageInfo
{
   IE_ImageInfo() { ZeroMemory(this, sizeof(*this)); dwReserved = 1; };

   DWORD       dwFlags;           // Flags -- set to 0
   DWORD       dwReserved;        // Reserved -- set to 1
};

/////////////////
// IE_PageInfo //
/////////////////
struct IE_PageInfo
{
   IE_PageInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD             dwFlags;           // IE_PIF_* flags
   BITMAPINFO*       pBMI;              // Pointer to BITMAPINFO struct (and colormap, if present)
   void*             pBits;             // Pointer to bitmap bits
   DWORD             dwPixFormatFlags;  // Pixel format flags
   void*             reserved_1;        // Internal use only
   void*             pAlpha;            // Pointer to alpha channel, where applicable
   int               nAlphaBPP;         // Alpha channel bpp.
   int               iTransparency;     // Transparent color index, where applicable
   int               nDelay;            // Frame delay in ms, where applicable
   void*             reserved_2;        // Internal use only
   void*             reserved_3;        // Internal use only
   void*             reserved_4;        // Internal use only
   MD_MetadataList*  pMetadataList;     // List of metadata to add to image
   BOOL              bHasTranparencyChannel; //Image has transparency channel
   char*             pParamACDDefinedXML;//ACDSee custom metadata
};


///////////////////
// IE_OutputInfo //
///////////////////
struct IE_OutputInfo
{
   IE_OutputInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD          dwFlags;           // Flags -- set to 0
   DWORD          dwFormatID;        // Output format ID
   const wchar_t* pszFN;             // Name of file to create
};

///////////////////////////
// IE_OutputInfoToBuffer //
///////////////////////////
//Added for encoding image to buffer
struct IE_OutputInfoToBuffer : public IE_OutputInfo
{
   IE_OutputInfoToBuffer() { ZeroMemory(this, sizeof(*this)); };

   void*          pOutputBuf;        // buffer to copy
   DWORD*         pSizeOut;          // size copied to buffer
};

////////////////////////
// PF_IE_ProgressProc //
////////////////////////
typedef int (WINAPI *PF_IE_Progress)
  (void* pParam,                 // User parameter
   int   nProgressNum,           // Progress numerator
   int   nProgressDen);          // Progress denominator

/////////////////////
// IE_EncodeParams //
/////////////////////
struct IE_EncodeParams
{
   IE_EncodeParams() { ZeroMemory(this, sizeof(*this)); };

   DWORD       dwSize;            // Total size, in bytes, of private encoding parameters structure
   DWORD       dwFlags;           // EPF_* flags.
                                  // Private, plug-in defined fields
};

/////////////////////
// IE_CallbackData //
/////////////////////
struct IE_CallbackData
{
   IE_CallbackData() : pfProgress(NULL), pParam(NULL) {};

   PF_IE_Progress  pfProgress;    // Progress callback proc
   void*           pParam;        // Parameter to pass back to callback proc
};

///////////////////////////
// IE_ErrorHandlerParams //
///////////////////////////
// Error Handler Parameters
struct IE_ErrorHandlerParams
{
   IE_ErrorHandlerParams() { ZeroMemory(this, sizeof(*this)); };

   BOOL				fIsGUI;     // Does the host app have a GUI
};

/////////////////
// IE_StateHdl //
/////////////////
typedef void* IE_StateHdl;

/////////////////////
// IE_MODE_* flags //
/////////////////////
// Encoding mode.
#define IE_MODE_CREATENEW       1     // Create a new image, encoding pages sequentially
#define IE_MODE_EDITEXISTING    2     // Edit an existing image, appending or replacing pages

///////////////////////
// Function pointers //
///////////////////////
typedef int(__stdcall *PF_IE_Init)                         (IE_ClientInfo*);
typedef int(__stdcall *PF_IE_Free)                         ();
typedef int(__stdcall *PF_IE_GetPlugInInfo)                (IE_PlugInInfo**);
typedef int(__stdcall *PF_IE_ShowPlugInDialog)             (HWND);
typedef int(__stdcall *PF_IE_GetError)                     (int, wchar_t*, int);
typedef int(__stdcall *PF_IE_GetDefaultEncodeParams)       (DWORD, int*, IE_EncodeParams*);
typedef int(__stdcall *PF_IE_ShowEncodeParamsDialog)       (DWORD, HWND, IE_EncodeParams*);
typedef int(__stdcall *PF_IE_GetValidPixelFormats)         (DWORD, IE_EncodeParams*, int*, DWORD*);
typedef int(__stdcall *PF_IE_ImageEncode)                  (IE_OutputInfo*, IE_EncodeParams*, IE_ImageInfo*, IE_PageInfo*, IE_CallbackData*);
typedef int(__stdcall *PF_IE_ImageEncodeToBuffer)          (IE_OutputInfoToBuffer*, IE_EncodeParams*, IE_ImageInfo*, IE_PageInfo*, IE_CallbackData*);
typedef int(__stdcall *PF_IE_ImageEncodeStart)             (int iMode, IE_OutputInfo*, IE_ImageInfo*, IE_StateHdl*);
typedef int(__stdcall *PF_IE_ImageEncodePage)              (IE_StateHdl, int iPage, IE_EncodeParams*, IE_PageInfo*, IE_CallbackData*);
typedef int(__stdcall *PF_IE_ImageEncodeFinish)            (IE_StateHdl, IE_CallbackData*);
typedef int(__stdcall *PF_IE_Help)                         (HWND, UINT, DWORD);
typedef int(__stdcall *PF_IE_ErrorHandler)                 (IE_ErrorHandlerParams*);
typedef int(__stdcall *PF_IE_FormatInfoChanged)            (BOOL&);
typedef int(__stdcall *PF_IE_SetMetadataList)              (const wchar_t*, int, MD_MetadataList*, IE_CallbackData*);
typedef int(__stdcall *PF_IE_GetSupportedMetadata)         (DWORD dwFormatID, int&, GUID**);
typedef int(__stdcall *PF_IE_GetNativelySupportedMetadata) (DWORD dwFormatID, int&, GUID**);
typedef int(__stdcall *PF_IE_SetEncodingInfo)              (void*   pEncodingInfo);

/**********
* IE_Init *
***********
* The client application will call this function once after loading the
* plug-in library. The plug-in must not maintain the pointer to
* IE_ClientInfo, it should make a copy the information in the structure.
*/
PLUGIN_API int __stdcall IE_Init(
   IE_ClientInfo* pci); // (in) Information about the plug-in client (host)

/**********
* IE_Free *
***********
* The client application will call this function once when it no longer
* needs the plug-in.  After IE_Free() is called, no other entry points
* may be called.  Normally, IE_Free() will be called immediately before
* unloading the plug-in DLL using FreeLibrary().
*
* IE_Free() will not be called if IE_Init() failed.
*
* The main purpose of IE_Free() is to give the plug-in a chance to free
* any DLLs it has dynamically loaded with LoadLibrary().
*
* Returns: IEE_OK.
*/
PLUGIN_API int __stdcall IE_Free();

/*******************
* IE_GetPlugInInfo *
********************
* Returns a pointer to an IE_PlugInInfo.
* The plug-in owns the pointer and must not free it until the
* plug-in library is unloaded.
*/
PLUGIN_API int __stdcall IE_GetPlugInInfo(
   IE_PlugInInfo**   pi); // (out) Plug-in information

/**********************
* IE_ShowPlugInDialog *
***********************
* Shows an "About" and/or "Options" dialog for this plug-in.
*/
PLUGIN_API int __stdcall IE_ShowPlugInDialog(
   HWND              hwndParent);     // (in ) Parent window

/**************
* IE_GetError *
***************
* Loads pszErrBuf with a null-terminated string describing the given
* plug-in specific error code.
*/
PLUGIN_API int __stdcall IE_GetError(
   int               nError,          // (in ) Error code
   wchar_t*          pszErrBuf,       // (out) Buffer to put error message text info
   int               cbErrBuf);       // (in ) Size of error buffer

/****************************
* IE_GetDefaultEncodeParams *
*****************************
* Initializes the encoding parameters struct for the specified format to default values.
* Upon entry, pcbEncodeParams gives the size of pEncodeParams.
*
* The plug-in should handle the following cases:
* 1) pcbEncodeParams too small:  Set pcbEncodeParams to the correct size and return IEE_StructSize.
* 2) pEncodeParams is NULL:      Set pcbEncodeParams to the correct size and return IEE_InvalidParam.
* 3) pcbEncodeParams is large
*    enough and pEncodeParams
*    is not NULL:                Initialize pEncodeParams to default values.
*/
PLUGIN_API int __stdcall IE_GetDefaultEncodeParams(
   DWORD             dwFormatID,      // (in ) ID of format to get encoding parameters for
   int*              pcbEncodeParams, // (i/o) Size of encoding parameters
   IE_EncodeParams*  pEncodeParams);  // (out) Receives default encoding parameters

/****************************
* IE_ShowEncodeParamsDialog *
*****************************
* Displays a dialog box allowing the user to customize the encoding options
* for the specified format.  The dialog box should be initialized to reflect
* the input settings of IE_EncodeParams.  If the user clicks on OK, the
* plug-in should change IE_EncodeParams to reflect the dialog box settings
* and return IEE_OK.  If the user clicks on Cancel, then the plug-in should
* leave not change IE_EncodeParams, and return IEE_Cancelled.  The dialog
* box should include a "save settings" checkbox, and when the user checks
* it and clicks on OK, the plug-in should set the EPF_SAVESETTINGS flag in
* IE_EncodeParams::dwFlags.
*/
PLUGIN_API int __stdcall IE_ShowEncodeParamsDialog(
   DWORD             dwFormatID,      // (in ) ID of format for which to show encoding parameters dialog
   HWND              hwndParent,      // (in ) Parent window
   IE_EncodeParams*  pEP);            // (i/o) Encoding parameters

/**************************
* IE_GetValidPixelFormats *
***************************
* Requests an enumeration of all pixel formats supported by the specified
* format and encoding parameters.
*
* Note: If *pnPixFormats is set to 20 by the caller, then "new" pixel formats
* (as designated in the comments for the definitions of PFF_*)
* must not be returned by the plug-in.  This is a workaround for a bug in versions
* of ACDSee prior to 7, which rejected file formats that returned unrecognized
* pixel formats.
*
*/
PLUGIN_API int __stdcall IE_GetValidPixelFormats(
   DWORD             dwFormatID,      // (in ) ID of format to get PixFormats for
   IE_EncodeParams*  pEP,             // (in ) Encoding parameters
   int*              pnPixFormats,    // (i/o) On entry, size of pPixFormats array.  On exit, #pixel valid formats.
   DWORD*            pPixFormats);    // (out) Array of valid pixel formats

/*****************
* IE_ImageEncode *
******************
* Writes a single-page image in one go.
*/
PLUGIN_API int __stdcall IE_ImageEncode(
   IE_OutputInfo*   pOI,             // (in ) Output information
   IE_EncodeParams* pEP,             // (in ) Encoding parameters
   IE_ImageInfo*    pII,             // (in ) Image information
   IE_PageInfo*     pPI,             // (in ) Page  information
   IE_CallbackData* pCD);            // (in ) Progress callback data

/*************************
* IE_ImageEncodeToBuffer *
**************************
* Writes a single-page image to buffer.
*/
PLUGIN_API int __stdcall IE_ImageEncodeToBuffer(
   IE_OutputInfoToBuffer*   pOI,             // (in ) Output information
   IE_EncodeParams*         pEP,             // (in ) Encoding parameters
   IE_ImageInfo*            pII,             // (in ) Image information
   IE_PageInfo*             pPI,             // (in ) Page  information
   IE_CallbackData*         pCD);            // (in ) Progress callback data

/**********************
* IE_ImageEncodeStart *
***********************
* Begins writing to a new or existing image file.
*
* IE_MODE_EDITEXISTING is supported only when the FIF_EDITMODE flag is specified in IE_FormatInfo.
* If IE_MODE_CREATENEW is specified, and the output image exists, it will be overwritten.
* If IE_MODE_EDITEXISTING is specified, then the output image must exist, or an error occurs.
*/
PLUGIN_API int __stdcall IE_ImageEncodeStart(
   int              iMode,           // (in ) Encoding mode: either IE_MODE_CREATENEW or IE_MODE_EDITEXISTING
   IE_OutputInfo*   pOI,             // (in ) Output information
   IE_ImageInfo*    pII,             // (in ) Image information
   IE_StateHdl*     phState);        // (out) Encoding state handle

/*********************
* IE_ImageEncodePage *
**********************
* Appends or replaces a page in the image file.
*
* If the encoding mode is IE_MODE_CREATENEW, then iPage must be set to the number
* of pages already written to the image (i.e., the index of the last page written plus one).
* Pages may only be appended sequentially in IE_MODE_CREATENEW mode.
*
* If the encoding mode is IE_MODE_EDITEXISTING, then iPage must be between 0 and the number
* of pages currently in the image.  If iPage refers to an existing page (0<=iPage<nPages), then
* the specified page is replaced by the specified page.  If iPage refers to a non-existing page
* (iPage==nPages), then the page is appended to the image.
*/
PLUGIN_API int __stdcall IE_ImageEncodePage(
   IE_StateHdl      hState,          // (in ) Image state handle
   int              iPage,           // (in ) Destination page # (0-based)
   IE_EncodeParams* pEP,             // (in ) Encoding parameters for this page
   IE_PageInfo*     pPI,             // (in ) Page data to be written
   IE_CallbackData* pCD);            // (in ) Progress callback data

/***********************
* IE_ImageEncodeFinish *
************************
* Closes the image file.
* Some plug-in impelmentations may require a significant amount of time to complete this operation,
* in which case the supplied progress callback function will be called to indicate the progress.
*/
PLUGIN_API int __stdcall IE_ImageEncodeFinish(
   IE_StateHdl      hState,          // (in ) Image state handle
   IE_CallbackData* pCD);            // (in ) Progress callback data

/*********************
* IE_SetMetadataList *
**********************/
//Sets metadata to a specific page of an image. 
PLUGIN_API int __stdcall IE_SetMetadataList(
   const wchar_t*     pszFileName,     // (in ) Filename
   int                iPage,           // (in ) Page number
   MD_MetadataList*   pList,           // (in ) Metadata List
   IE_CallbackData*   pCbData);        // (in ) Progress Callback Data

/**************************
* IE_GetSupportedMetadata *
***************************
* Gets the supported Metadata types of the format. pFormats should be assigned to a plug-in allocated array of 
* GUID's, representing the supported metadata types. 
*/
PLUGIN_API int __stdcall IE_GetSupportedMetadata(
   DWORD  dwFormatID,                  // (in ) Format ID
   int&   nFormats,                    // (out) Number of supported formats
   GUID** pFormats);                   // (out) GUIDs of supported formats

/**********************************
* IE_GetNativelySupportedMetadata *
***********************************
* Gets the supported Metadata types of the format. pFormats should be assigned to a plug-in allocated array of
* GUID's,representing the supported metadata types. There should be no custom metadata types added, only the image 
* format specification defined ones. 
*/
PLUGIN_API int __stdcall IE_GetNativelySupportedMetadata(
   DWORD,                              // (in ) Format ID
   int&,                               // (out) Number of supported formats
   GUID**);                            // (out) GUIDs of supported formats

/**********
* IE_Help *
***********
* Displays help for the plug-in or a specific format.
*/
PLUGIN_API int __stdcall IE_Help(
   HWND  hWnd,       // Application window that is calling help                                 
   UINT  uCommand,   // Type of help requested (IE_HC_* value)
   DWORD dwData);    // Additional data for help command

/******************
* IE_ErrorHandler *
*******************
* Handles an exception caused by the plug-in. The host will call this function if the
* plug-in causes an exception.
*
* The return value should be one of the IE_EHF_* flags.
*/

PLUGIN_API int __stdcall IE_ErrorHandler(
   IE_ErrorHandlerParams*    pehp);          // (in) Error Handler Parameters

/***********************
* IE_FormatInfoChanged *
************************
* Called by the plug-in host to determine whether or not to update any cached information about
* formats supported by this plug-in.
*
* Sets bChanged to TRUE if the host should update it's cache.
*/
PLUGIN_API int __stdcall IE_FormatInfoChanged(BOOL& bChanged);

#endif
