//-------------------------------------------------------------------
// IF_PlugIn.h
//-------------------------------------------------------------------
// ACD SYSTEMS INTERNATIONAL INC.
// Copyright © 1994 - 2020 ACD Systems International Inc.
// All Rights Reserved
//
// NOTICE: ACD Systems International Inc. ("ACD") permits you to use,
// modify, and distribute this file in accordance with the terms of
// the ACDSee SDK license agreement accompanying it. If you have 
// received this file from a source other than ACD, then your use,
// modification, or distribution of it requires the prior written 
// permission of ACD.
//-------------------------------------------------------------------

/**********************************************************************************

All functions return a status code of 0 on success, or a negative error code
if a failure occurs.

Required functions:

IF_Init
IF_GetPluginInfo

IF_RunFilter
OR
IF_RunMenuFilter
IF_RunMenuFilterAction
IF_FreeAllocatedParams

Optional functions:
IF_ShowPluginDialog
IF_Help
IF_ErrorHandler
IF_Free

**********************************************************************************/

#ifndef _IF_PLUGIN_H_INCLUDED
#define _IF_PLUGIN_H_INCLUDED

#include <windows.h>

#ifndef PLUGIN_API
#define PLUGIN_API extern "C"
#endif

/////////////////////
// Plug-in version //
/////////////////////
#define IF_VERSION 100


/////////////////
// CIF_* flags //
/////////////////
#define CIF_REGISTERED 1 // Calling application is registered

//////////////////
// Status codes //
//////////////////
//General IF_ API return codes.
enum
{
   IFE_OK                 =  0,  // Success
   IFE_Error              = -1,  // Unspecified error
   IFE_InvalidOp          = -2,  // Operation cannot be completed at this time
   IFE_InvalidParam       = -3,  // Invalid parameters
   IFE_NotImplemented     = -4,  // Missing implementation
   IFE_InvalidFilter      = -5,  // The specified filter index is invalid
   IFE_Cancelled          = -6,  // Operation cancelled by user
   IFE_ImageFilterError   = -7,  // The specified filter could not be applied
   IFE_Malloc             = -8,  // Memory allocation error
   IFE_Decode             = -9,  // Image decode error
   IFE_FatalError         = -10, // Unhandled exception
   IFE_HostVersion        = -11, // Host application does not support correct plug-in interface version
   IFE_FilterNotFound     = -12, // specified filter could not be found
                                 // other errors...

   // Plug-in defined errors start at -1000
   IFE_FirstPlugInError   = -1000,
};

///////////////////////////
// IF_HC_* Help Commands //
///////////////////////////
#define IF_HC_PLUGIN 1 // Displays help for the plug-in.
// dwData is not used.

#define IF_HC_FORMAT 2 // Displays help for a filter.
// dwData is the index of the format.

//////////////////////////
// IF_FILTER_TOOL flags //
//////////////////////////
//Specifies the active mouse tool for the filter. Used in IF_UISpecification.
//Output from the tools are specified in IF_FILTER_TOOL_PARAMS and passed to the plug-in
//via IF_FilterParams.
enum IF_FILTER_TOOL
{
   IF_TOOL_NONE,              // No mouse tool
   IF_TOOL_SELECTION,         // Selection tool
   IF_TOOL_DISTORT,           // Distortion tool
   IF_TOOL_POSITIONSELECT,    // Position Selection tool
   IF_TOOL_COLORSELECT,       // Color Selection tool
};

///////////////////////////
// IF_FILTER_TOOL_PARAMS //
///////////////////////////
//Identifiers of parameters passed to the filter from the mouse tool specified in IF_FILTER_TOOl.
//The nParamID field of the IF_SingleParam structure will be set to these flags.
enum IF_FILTER_TOOL_PARAMS
{
   //Coordinates of the four corners of the selection tool
   IF_TOOL_SELECTION_PARAM_TOPLEFT_X = 1000, //int
   IF_TOOL_SELECTION_PARAM_TOPLEFT_Y,        //int
   IF_TOOL_SELECTION_PARAM_TOPRIGHT_X,       //int
   IF_TOOL_SELECTION_PARAM_TOPRIGHT_Y,       //int
   IF_TOOL_SELECTION_PARAM_BOTTOMLEFT_X,     //int
   IF_TOOL_SELECTION_PARAM_BOTTOMLEFT_Y,     //int
   IF_TOOL_SELECTION_PARAM_BOTTOMRIGHT_X,    //int
   IF_TOOL_SELECTION_PARAM_BOTTOMRIGHT_Y,    //int

   //Coordinates of the four corners of the distortion tool
   IF_TOOL_DISTORT_PARAM_TOPLEFT_X,          //int
   IF_TOOL_DISTORT_PARAM_TOPLEFT_Y,          //int
   IF_TOOL_DISTORT_PARAM_TOPRIGHT_X,         //int
   IF_TOOL_DISTORT_PARAM_TOPRIGHT_Y,         //int
   IF_TOOL_DISTORT_PARAM_BOTTOMLEFT_X,       //int
   IF_TOOL_DISTORT_PARAM_BOTTOMLEFT_Y,       //int
   IF_TOOL_DISTORT_PARAM_BOTTOMRIGHT_X,      //int
   IF_TOOL_DISTORT_PARAM_BOTTOMRIGHT_Y,      //int

   //Position of the selection for position select tool
   IF_TOOL_POSITIONSELECT_PARAM_X,           //int
   IF_TOOL_POSITIONSELECT_PARAM_Y,           //int

   //Color value selected from the color select tool.
   IF_TOOL_COLORSELECT_PARAM_COLORREF,       //DWORD

};
//////////////////////
// IF_UI_MAX limits //
//////////////////////
//Hard Limits to amounts of each type of UI control you can ask ACDSee for
enum
{
   IF_UI_MAX_SLIDERS = 15,
   IF_UI_MAX_BUTTONGROUPS = 5,
   IF_UI_MAX_ANGLEPICKERS = 5,
   IF_UI_MAX_COLORPICKERS = 5,
};

///////////////////////////////////////
// IF_FILTER_DIALOG_CAPABILITY flags //
///////////////////////////////////////
//Dialog flags to customize the UI for a Filter
enum IF_FILTER_DIALOG_CAPABILITY
{
    IF_FDBD_HASPOSSELECT                        = (1 << 4),  // Filter has a selectable centre
    IF_FDBD_HASPROGRESS                         = (1 << 5),  // Filter dialog should show a progress window when previewing
    IF_FDBD_HASAPPLY                            = (1 << 6),  // Filter dialog has an apply button.
    IF_FDBD_NOMULTISUPPORT                      = (1 << 7),  // Filter should not be used on multiple images
    IF_FDBD_NEEDSLARGEDIALOG                    = (1 << 8),  // Filter dialog needs to be expanded vertically for more room
    IF_FDBD_PROGRESSFORPREVIEW                  = (1 << 9),  // After rendering for preview requires progress UI
    IF_FDBD_NODIFFBUTTON                        = (1 << 10), // Filter dialog has no Diff button.
    IF_FDBD_NOCANCELBUTTON                      = (1 << 11), // Filter dialog has no Cancel button
    IF_FDBD_NOMODALPROGRESS                     = (1 << 12), // Host application should not show a modal progress dialog when rendering the filter
    IF_FDBD_CANTUSESELECTIONS                   = (1 << 13), // Filter does not work well with selections.
    IF_FDBD_DEFAULTTOLASTUSED                   = (1 << 14), // Filter dialog will default to the Last Used preset (if there is one).
    IF_FDBD_NOT_AN_IMAGE_FILTER                 = (1 << 15), // Means this isn't really an image filter that you want applied to an image, like the Effects Browser.
    IF_FDBD_NO_RESET_BUTTON                     = (1 << 16), // Means this dialog doesn't have a reset button.
    IF_FDBD_HAS_PREVIEW_HISTOGRAM               = (1 << 17), // Filter is capable of showing a histogram of the adjusted image
    IF_FDBD_HAS_UNDO_BUTTONS                    = (1 << 18), // Filter should show the undo/redo buttons
    IF_FDBD_SHOWS_SAVED_INSTEAD_OF_PREVIOUS     = (1 << 19), // Filter should show the saved image when the Diff button is pushed
    IF_FDBD_NEEDS_FLATTENED_PREVIEW__SIZE_IMAGE = (1 << 20), // Some dialogs show the flattened image as part of the dialog preview (Channel Mixer)
    IF_FDBD_NO_PIXEL_TARGETING                  = (1 << 21), // Pixel targeting not allowed.  This only needs to be specified for filters that support blend modes but not pixel targeting
    IF_FDBD_ARCHIVE_CAN_BE_FILTERED             = (1 << 22), // This capability is used for non-destructive move operations
    IF_FDBD_FILTER_NEEDS_CAMERA_SETTINGS        = (1 << 23), // This fitler requires specific exif camera information (right now only Lens Correction with profiles).
};

//////////////////////////////////
// IF_FILTER_PREVIEW_CAPABILITY //
//////////////////////////////////
// Preview capability flags
enum IF_FILTER_PREVIEW_CAPABILITY
{
   IF_FDBP_CANZOOM                      = (1 << 0),  // Before/after windows are zoomable/scrollable
   IF_FDBP_CANRENDERROI                 = (1 << 1),  // Filter can render a rectangular region-of-interest
   IF_FDBP_CANRENDERSCALED              = (1 << 2),  // Filter can render scaled-down version of the image
   IF_FDBP_APPLY_TO_ALL_LAYERS          = (1 << 3),  // If layers are enabled, this filter must be applied to all layers
   IF_FDBP_LAYEREDFILTER                = (1 << 4),  // Prevents preview rendering because the layer takes care of it
   IF_FDBP_ALTERSMASKDIB                = (1 << 5),  // The mask DIB needs to be altered after running the filter. Rotate, for example.
   IF_FDBP_NEEDSEXACTBITDEPTH           = (1 << 6),  // The filter won't run correctly on a 24 bit version of a 64 bit image.
   IF_FDBP_HASFASTRENDER                = (1 << 7),  // The filter can run in a faster, lower quality mode.
   IF_FDBP_SUPPORTSHRUNKENIMAGES        = (1 << 8),  // The filter can run on scaled down images for increased speed.
   //                                                   Currently not compatible with filters that have caches and ROI.
   IF_FDBP_HASHIGHQUALITYDEMOSAIKRENDER = (1 << 9),  //The filter can render a high quality demosaiked version.
   IF_FDBP_NORFESUPPORT                 = (1 << 10), // The framework should not call GetCurrentConfig() to perform redundant filtering elimination
   IF_FDBP_PREVIEWRENDEREDBYVIEWTOOL    = (1 << 11), // The preview for this filter is rendered by a view tool
   IF_FDBP_CHANGES_IMAGE_SIZE           = (1 << 13), // The filter will change the size of the image
   IF_FDBP_FORCE_RENDER_SCALE           = (1 << 14), // The filter should always rerender when changing scale.
};


////////////////////
// IF_EHF_* flags //
////////////////////
// Error Handler Flags
enum
{
   IF_EHF_DEFAULT      = 1, // Activate default host Error Handler
   IF_EHF_DISABLE_PERM = 2, // Plug-in should be disabled by host app (permanently)
   IF_EHF_DISABLE_TEMP = 4, // Plug-in should be disabled by host app (temporarily)
};

///////////////
// IF_Bitmap //
///////////////
//See Windows Bitmap Documentation https://docs.microsoft.com/en-us/windows/desktop/gdi/bitmaps
//In addition, 48 bit bitmaps are 16 bits BGR and 64 bit bitmaps are 16 bits BGRA 
struct IF_Bitmap
{
   IF_Bitmap() { ZeroMemory(this, sizeof(*this)); };

   BITMAPINFO* pHeader;    //Pointer to Bitmap Header. Do not delete.
   void* pBits;            //Bit Pointer. Do not delete.
   int bytesPerRow;        //Bytes per row of bitmap. ACDSee sometimes aligns bitmaps and sometimes doesn't. Use this instead of calculating your own.
};

/////////////
// IF_Rect //
/////////////
// Basic rectangle
struct IF_Rect
{
   IF_Rect() { ZeroMemory(this, sizeof(*this)); };

   int left;
   int top;
   int right;
   int bottom;
};

/////////////
// IF_Size //
/////////////
// Basic Size
struct IF_Size
{
   IF_Size() { ZeroMemory(this, sizeof(*this)); };

   int cx;
   int cy;
};
///////////////////
// IF_RenderInfo //
///////////////////
struct IF_RenderInfo
{
   IF_RenderInfo() { ZeroMemory(this, sizeof(*this)); };

   IF_Size szFullOriginalImage; //Dimensions of the full, uncropped, unscaled original image
   IF_Size szFullScaledImage;   //Dimensinos of the full, uncropped, scaled version of the image.  Note that this can be the same as szFullOriginalImage;
   bool  fIsROI;                //Only TRUE when the entire image isn't visible on the screen.  
                                //Note that if the image is scaled down so the entire image fits the screen, this will be FALSE.
   IF_Rect rcROI;               //ROI rect
};

////////////////////
// IF_SingleParam //
////////////////////
// Represents a single Parameter from a UI element.
// Identified by unique ID that Plug-In creator assigns to the UI element.
struct IF_SingleParam
{
   IF_SingleParam() { ZeroMemory(this, sizeof(*this)); };

   UINT nParamID;           //Parameter ID, such as nSliderID. 1000-1100 are reserved for API defined parameters.
   int nParamLength;        //length of the parameter string
   wchar_t* pszParamValue;  //Parameter string, parse it based on parameter type. Do not delete.
};

/////////////////////
// IF_FilterParams //
/////////////////////
// Used to pass parameters to/from the plug-in filter
struct IF_FilterParams
{
   IF_FilterParams() { ZeroMemory(this, sizeof(*this)); };

   int nParams;             //Number of parameters
   IF_SingleParam* pParams; //Array of Parameters passed in to RunFilter. Only delete in IF_FreeAllocatedParams.
};

///////////////////
// IF_ClientInfo //
///////////////////
struct IF_ClientInfo
{
   IF_ClientInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD    dwFlags;       // CIF_* flags
   wchar_t  szCompany[40]; // Name of company of calling application
   wchar_t  szAppName[40]; // Name of calling application.
};

///////////////////
// IF_ButtonInfo //
///////////////////
//Represents a single Radio/Check Button
struct IF_ButtonInfo
{
   IF_ButtonInfo() { ZeroMemory(this, sizeof(*this)); };

   wchar_t szText[256]; // Display Text
   UINT nButtonID;      // Unique Button Identifier
   BOOL bDefaultState;  // Does the button start checked
};

////////////////////////
// IF_ButtonGroupInfo //
////////////////////////
//Represents a group of 1-3 Radio/Check Buttons
struct IF_ButtonGroupInfo
{
   IF_ButtonGroupInfo() { ZeroMemory(this, sizeof(*this)); };

   wchar_t szTitle[256];       //Display Text
   UINT nButtonGroupID;        //Unique Button Group Identifier
   BOOL bIsRadio;              //Is this a radio group
   int nButtons;               //Max 3
   IF_ButtonInfo* pButtonInfo; //Array of IF_ButtonInfo, with nButtons elements.
};

////////////////////////
// IF_ColorPickerInfo //
////////////////////////
//Represents a ColorPicker Control with an attached Enable/Disable Toggle Button
struct IF_ColorPickerInfo
{
   IF_ColorPickerInfo() { ZeroMemory(this, sizeof(*this)); };

   wchar_t szTitle[256];            //Display Text
   UINT nColorPickerID;             //Unique Color Picker Identifier
   COLORREF defaultColor;           //Default color

   BOOL bHasToggleButton;           //Is the color picker toggleable
   wchar_t szToggleButtonText[256]; //Display Text of the toggle button
   BOOL bToggleButtonDefaultState;  //Default state of the toggle button
};

////////////////////////
// IF_AnglePickerInfo //
////////////////////////
//Represents an AnglePicker Control
struct IF_AnglePickerInfo
{
   IF_AnglePickerInfo() { ZeroMemory(this, sizeof(*this)); };

   wchar_t szTitle[256];     //Display Text
   UINT nAnglePickerID;      //Unique Angle Picker Identifier
   float fDefaultPos;        //Starting Angle
};

///////////////////
// IF_SliderInfo //
///////////////////
//Represents a Slider Control
struct IF_SliderInfo
{
   IF_SliderInfo() { ZeroMemory(this, sizeof(*this)); };

   wchar_t szTitle[256];    //Display Name of the Slider
   UINT    nSliderID;       //Unique ID for the slider, used to pass params to the plug-in
   float   fMin;            //Minimum Displayed value.
   float   fMax;            //Maximum Displayed value.
   float   fStartVal;       //The starting position for this slider.
   float   fStep;           //How far one tick on slider goes
   BOOL    bHasEditBox;     //Does the slider have an edit box that lets the user enter values
   BOOL    bHasBorder;      //Does the slider have a border drawn around it
};

////////////////////////
// IF_UISpecification //
////////////////////////
//Represents the entire UI Specification that ACDSee should display for your filter.
// The plug-in owns the memory pointed to by pSliderInfo, pAnglePickerInfo, pColorPickerInfo
// and pButtonGroupInfo and must not free it until the plug-in library is unloaded.
struct IF_UISpecification
{
   IF_UISpecification() { ZeroMemory(this, sizeof(*this)); };

   DWORD dialogFlags;   //IF_FILTER_DIALOG_CAPABILITY flags
   DWORD previewFlags;  //IF_FILTER_PREVIEW_CAPABILITY flags

   IF_FILTER_TOOL filterTool;              //Which tool (or none) that will be active for the filter

   BOOL bSquishUI;                         //Should UI be squished (less space between elements)

   int nSliders;                           //Number of sliders
   IF_SliderInfo* pSliderInfo;             //Array of slider specifications

   int nAnglePickers;                      //Number of angle pickers
   IF_AnglePickerInfo* pAnglePickerInfo;   //Array of angle picker specifications

   int nColorPickers;                      //Number of color pickers
   IF_ColorPickerInfo* pColorPickerInfo;   //Array of color picker specifications

   int nButtonGroups;                      //Number of button groups
   IF_ButtonGroupInfo* pButtonGroupInfo;   //Array of button group specifications
};


///////////////////
// IF_FilterInfo //
///////////////////
// Specifies information about an image filter supported by the plug-in.
struct IF_FilterInfo
{
   IF_FilterInfo() { ZeroMemory(this, sizeof(*this)); };

   DWORD    dwFlags;           // Flags -- set to 0
   DWORD    dwID;              // Unique identifier for this filter
   wchar_t  szName[40];        // Name of this Filter
   wchar_t  szNameShort[8];    // Short name of this Filter
   BOOL     bIsNativeGUIFilter;// Does this plugin support native ACDSee GUI. If true, requires uiSpec, and filter appears with other GUI filters. If false, filter will appear in a menu.
   UINT     iIcon;             // Filter icon resource id
   IF_UISpecification uiSpec;  // UI Specification for the Filter. Only valid when bIsNativeGUIFilter is set
};

///////////////////
// IF_PlugInInfo //
///////////////////
// Specifies information about the plug-in.
// The plug-in owns the memory pointed to by pFilterInfo and must not free it until
// the plug-in library is unloaded.
struct IF_PlugInInfo
{
   IF_PlugInInfo() { ZeroMemory(this, sizeof(*this)); };
   DWORD          dwFlags;     // Flags -- set to 0
   GUID           guid;        // Unique Identifier for the plugin - used to identify the plugin for actions. You should update this each version.
   int            nVersion;    // Plug-in specificiation version
   wchar_t        szTitle[40]; // Plug-in title
   UINT           iIcon;       // Plug-in icon resource id
   int            nFilters;    // Number of filters supported by plug-in
   IF_FilterInfo* pFilterInfo; // Information about each filter supported
};

///////////////////////////
// IF_ErrorHandlerParams //
///////////////////////////
// Error Handler Parameters
struct IF_ErrorHandlerParams
{
   IF_ErrorHandlerParams() { ZeroMemory(this, sizeof(*this)); };

   BOOL				fIsGUI;     // Does the host app have a GUI
};

///////////////////////
// Function pointers //
///////////////////////
typedef int(__stdcall *PF_IF_Init)                (IF_ClientInfo*);
typedef int(__stdcall *PF_IF_Free)                ();
typedef int(__stdcall *PF_IF_GetPlugInInfo)       (IF_PlugInInfo**);
typedef int(__stdcall *PF_IF_ShowPlugInDialog)    (HWND);
typedef int(__stdcall *PF_IF_RunFilter   )        (IF_Bitmap*, IF_RenderInfo*, DWORD, IF_FilterParams*);
typedef int(__stdcall *PF_IF_RunMenuFilter)       (HWND, IF_Bitmap*, IF_Bitmap*, DWORD, IF_FilterParams**);
typedef int(__stdcall *PF_IF_RunMenuFilterAction) (IF_Bitmap*, IF_Bitmap*, DWORD, IF_FilterParams*);
typedef int(__stdcall *PF_IF_FreeAllocatedParams) (IF_FilterParams*);
typedef int(__stdcall *PF_IF_Help)                (HWND, UINT, DWORD);
typedef int(__stdcall *PF_IF_ErrorHandler)        (IF_ErrorHandlerParams*);

/**********
* IF_Init *
***********
* The host application will call this function once after loading the plug-in library.
* The plug-in must not maintain the pointer to IF_ClientInfo, it should make a copy of
* the information in the structure.
*/
PLUGIN_API int __stdcall IF_Init(
   IF_ClientInfo* pci); // (in) Information about the plug-in client (host)

/**********
* IF_Free *
***********
* The client application will call this function once when it no longer
* needs the plug-in.  After IF_Free() is called, no other entry points
* may be called.  Normally, IF_Free() will be called immediately before
* unloading the plug-in DLL using FreeLibrary().
*
* IF_Free() will not be called if IF_Init() failed.
*
* The main purpose of IF_Free() is to give the plug-in a chance to free
* any DLLs it has dynamically loaded with LoadLibrary().
*
* Returns: IFE_OK.
*/
PLUGIN_API int __stdcall IF_Free();

/*******************
* IF_GetPlugInInfo *
********************
* Returns a pointer to an IF_PlugInInfo.
* The plug-in owns the pointer and must not free it until the
* plug-in library is unloaded.
*/
PLUGIN_API int __stdcall IF_GetPlugInInfo(
   IF_PlugInInfo** ppii); // (out) filter Plug-in information

/**********************
* IF_ShowPlugInDialog *
***********************
* Shows an "About" and/or "Options" dialog for this plug-in.
*/
PLUGIN_API int __stdcall IF_ShowPlugInDialog(
   HWND  hwndParent);            // (in) Parent window

/***************
* IF_RunFilter *
****************
*  Runs an ACDSee GUI based filter on a bitmap, given the parameters set via ACDSee's GUI, that were specified in
*  the IF_UISpecification for the filter.
*/
PLUGIN_API int __stdcall IF_RunFilter(
   IF_Bitmap*       pBitmap,      // [i/o] image bitmap to run filter on. Modify in place to apply filter
   IF_RenderInfo*   pRenderInfo,  // [in]  information about image to render
   DWORD            dwFilterID,   // [in]  which filter to apply
   IF_FilterParams* pParams);     // [in]  parameters to run the filter with.

/*******************55
* IF_RunMenuFilter *
********************
*  Runs a filter from a menu. Any UI must be fully implemented by the plug-in. If the user set parameters for the filter,
*  return those parameters via outParams so that ACDSee can re-run the filter via actions later.
*/
PLUGIN_API int __stdcall IF_RunMenuFilter(
   HWND              hwndParent,   // [in] Parent window
   IF_Bitmap*        pBitmap,      // [i/o] image bitmap to run filter on. Modify in place to apply filter.
   IF_Bitmap*        pMask,        // [in] selection mask for applying filter
   DWORD             dwFilterID,   // [in] which filter to apply
   IF_FilterParams** ppOutParams); // [out] params that were used to apply the filter to image. Used to return back to plug-in for actions support.

/*************************
* IF_RunMenuFilterAction *
**************************
* Runs a menu filter during an action playback.
*/
PLUGIN_API int __stdcall IF_RunMenuFilterAction(
   IF_Bitmap*       pBitmap,    // [i/o] image bitmap to run filter on. Modify in place to apply filter.
   IF_Bitmap*       pMask,      // [in] selection mask for applying filter
   DWORD            dwFilterID, // [in] which filter to apply
   IF_FilterParams* ppParams);  // [in] parameters for the filter. Identical to the params passed back from IF_RunMenuFilter.

/*************************
* IF_FreeAllocatedParams *
**************************
*  Frees the parameters returned via IF_RunMenuFilter. ACDSee will have made a copy of those parameters for future use.
*/
PLUGIN_API int __stdcall IF_FreeAllocatedParams(
   IF_FilterParams* pParams); // [in] Params to deallocate. Plugin should free this memory.

/**********
* IF_Help *
***********
* Displays help for the plug-in or a specific format.
*/
PLUGIN_API int __stdcall IF_Help(
    HWND        hWnd,       // Application window that is calling help
    UINT        uFilter,    // Type of help requested (IF_HC_* value)
    DWORD       dwData);    // Additional data for help command

/******************
* IF_ErrorHandler *
*******************
* Handles an exception caused by the plug-in. The host will call this function if the
* plug-in causes an exception.
*
* The return value should be one of the IF_EHF_* flags
*/
PLUGIN_API int __stdcall IF_ErrorHandler(
      IF_ErrorHandlerParams*    pehp);          // (in) Error Handler Parameters

#endif
