//-------------------------------------------------------------------
// Exif.h
//-------------------------------------------------------------------
// ACD SYSTEMS INTERNATIONAL INC.
// Copyright © 1994 - 2020 ACD Systems International Inc.
// All Rights Reserved
//
// NOTICE: ACD Systems International Inc. ("ACD") permits you to use,
// modify, and distribute this file in accordance with the terms of
// the ACDSee SDK license agreement accompanying it. If you have 
// received this file from a source other than ACD, then your use,
// modification, or distribution of it requires the prior written 
// permission of ACD.
//-------------------------------------------------------------------

/**********************************************************************************

 Defines an "extended data table" for holding metadata information (such as 
 Exif info) about an image file.  Its purpose is to allow an image decoding 
 plug-in to provide metadata contained in an image file to a client application,
 and allow a client application to pass metadata to an image encoding plug-in.
 Each data tag contains an ID for the data item, the data type, and the data itelf.
   
 **********************************************************************************/

#include <windows.h>

#ifndef _METADATA_H_INCLUDED
#define _METADATA_H_INCLUDED

/////////////
// MD_Item //
/////////////
struct MD_Item
{
   MD_Item() : nDataID(0), nTypeID(0), nCount(0), dwData(0), pNext(NULL) {};

   WORD   nDataID;     // Data item ID
   WORD   nTypeID;     // Data item type
   UINT   nCount;      // Number of values of the indicated type
   union
   {
      void*  pData;    // Data as void pointer
      char*  pszData;  // Data as string
      DWORD  dwData;   // Data as a 32-bit value
      BYTE   aData[4]; // Data as an array of four bytes
      SHORT  sData[2]; // Data as an array of two shorts
   };
   MD_Item* pNext;
};

#ifndef MDT_DEFINED
#define MDT_DEFINED

//////////////////////
// Data type values //
//////////////////////
enum
{
   MDT_BYTE      =  1, // 8-bit  unsigned integer
   MDT_STRING    =  2, // 8-bit byte that contains a NULL-terminated string
   MDT_SHORT     =  3, // 16-bit unsigned integer
   MDT_LONG      =  4, // 32-bit unsigned integer
   MDT_RATIONAL  =  5, // Two 32-bit unsigned integers; the first represent the numerator of a fraction; the second, the denominator
   MDT_SBYTE     =  6, // 8-bit  signed integer
   MDT_UNDEFINED =  7, // 8-bit byte that may contain anything, depending on the definition of the field
   MDT_SSHORT    =  8, // 16-bit signed integer
   MDT_SLONG     =  9, // 32-bit signed integer
   MDT_SRATIONAL = 10, // Two 32-bit signed integers; the first represent the numerator of a fraction; the second, the denominator
   MDT_FLOAT     = 11, // Single precision (32-bit) IEEE format floating point
   MDT_DOUBLE    = 12, // Double precision (64-bit) IEEE format floating point
};

static const int MD_NumTypes = 12;
static const int MD_TypeSizeTable[MD_NumTypes+1] = { 0, 1, 1, 2, 4, 8, 1, 1, 2, 4, 8, 4, 8 };

///////////////////
// Tag data ID's //
///////////////////
enum
{
   // Info tags - not part of Exif tags!
   MDT_CAPTION               =  1, // Deprecated
   MDT_PANASONICWIDTH        =  2,
   MDT_PANASONICHEIGHT       =  3,
   MDT_KEYWORDS              =  5, // Deprecated
   MDT_PAGENUMBER            =  8, // Deprecated
   MDT_COMMENT               = 11, // Deprecated

   // TIFF Tags (Exif level 1)
   MDT_LEVEL1_BEGIN		     = 0x0100,
   MDT_IMAGEDESCRIPTION      = 0x010e,
   MDT_MAKE                  = 0x010f,
   MDT_MODEL                 = 0x0110,
   MDT_ORIENTATION           = 0x0112,
   MDT_XRESOLUTION           = 0x011a,
   MDT_YRESOLUTION           = 0x011b,
   MDT_RESOLUTIONUNIT        = 0x0128,
   MDT_SOFTWARE              = 0x0131,
   MDT_DATETIME              = 0x0132,
   MDT_ARTIST                = 0x013b,
   MDT_WHITEPOINT            = 0x013e,
   MDT_PRIMARYCHROMATICITIES = 0x013f,
   MDT_YCBCRCOEFFICIENTS     = 0x0211,
   MDT_YCBCRPOSITIONING      = 0x0213,
   MDT_REFERENCEBLACKWHITE   = 0x0214,
   MDT_COPYRIGHT             = 0x8298,
   MDT_DNGPRIVATEDATA        = 0xc634,
                               
   // Exif private tags (Exif level 2)

   MDT_LEVEL2_BEGIN		     = 0x829a,
   MDT_EXPOSURETIME          = 0x829a,
   MDT_FNUMBER               = 0x829d,
   MDT_EXPOSUREPROGRAM       = 0x8822,
   MDT_SPECTRALSENSITIVITY   = 0x8824,
   MDT_ISOSPEEDRATINGS       = 0x8827,
   MDT_OPTOELECTRICCOEFF     = 0x8828,

   //Exif 2.3
   MDT_SENSITIVITYTYPE           = 0x8830,
   MDT_STANDARDOUTPUTSENSITIVITY = 0x8831,
   MDT_RECOMMENDEDEXPOSUREINDEX  = 0x8832,
   MDT_ISOSPEED                  = 0x8833,
   MDT_ISOSPEEDLATITUDEYYY       = 0x8834,
   MDT_ISOSPEEDLATITUDEZZZ       = 0x8835,

   MDT_EXIFVERSION           = 0x9000,
   MDT_DATETIMEORIGINAL      = 0x9003,
   MDT_DATETIMEDIGITIZED     = 0x9004,
   MDT_COMPONENTCONFIG       = 0x9101,
   MDT_COMPRESSEDBPP         = 0x9102,
   MDT_SHUTTERSPEEDVALUE     = 0x9201,
   MDT_APERTUREVALUE         = 0x9202,
   MDT_BRIGHTNESSVALUE       = 0x9203,
   MDT_EXPOSUREBIASVALUE     = 0x9204,
   MDT_MAXAPERTUREVALUE      = 0x9205,
   MDT_SUBJECTDISTANCE       = 0x9206,
   MDT_METERINGMODE          = 0x9207,
   MDT_LIGHTSOURCE           = 0x9208,
   MDT_FLASH                 = 0x9209,
   MDT_FOCALLENGTH           = 0x920a,
   MDT_MAKERNOTE             = 0x927c,
   MDT_USERCOMMENT           = 0x9286,
   MDT_SUBSECTIME            = 0x9290,
   MDT_SUBSECTIMEORIGINAL    = 0x9291,
   MDT_SUBSECTIMEDIGITIZED   = 0x9292,
   MDT_FLASHPIXVERSION       = 0xa000,
   MDT_COLORSPACE            = 0xa001,
   MDT_PIXELXDIMENSION       = 0xa002,
   MDT_PIXELYDIMENSION       = 0xa003,
   MDT_RELATEDSOUNDFILE      = 0xa004,
   MDT_INTEROPERABILITYIFD   = 0xa005,
   MDT_FLASHENERGY           = 0xa20b,
   MDT_SPATIALFREQRESPONSE   = 0xa20c,
   MDT_FOCALPLANEXRES        = 0xa20e,
   MDT_FOCALPLANEYRES        = 0xa20f,
   MDT_FOCALPLANERESUNIT     = 0xa210,
   MDT_SUBJECTLOCATION       = 0xa214,
   MDT_EXPOSUREINDEX         = 0xa215,
   MDT_SENSINGMETHOD         = 0xa217,
   MDT_FILESOURCE            = 0xa300,
   MDT_SCENETYPE             = 0xa301,
   MDT_CFAPATTERN            = 0xa302,

   // new Exif tags for Exif 2.2
   MDT_SUBJECTAREA           = 0x9214,
   MDT_CUSTOMRENDERED        = 0xa401,
   MDT_EXPOSUREMODE          = 0xa402,
   MDT_WHITEBALANCE          = 0xa403,
   MDT_DIGITALZOOMRATIO      = 0xa404,
   MDT_FOCALLENGTHIN35MMFILM = 0xa405,
   MDT_SCENECAPTURETYPE      = 0xa406,
   MDT_GAINCONTROL           = 0xa407,
   MDT_CONTRAST              = 0xa408,
   MDT_SATURATION            = 0xa409,
   MDT_SHARPNESS             = 0xa40a,
   MDT_DEVICESETTINGDESCRIPTION = 0xa40b,
   MDT_SUBJECTDISTANCERANGE  = 0xa40c,
   MDT_IMAGEUNIQUEID         = 0xa420,
   MDT_GAMMA                 = 0xa500,  //EXIF 2.21

   //exif 2.3
   MDT_CAMERAOWNERNAME       = 0xa430,
   MDT_BODYSERIALNUMBER        = 0xa431,
   MDT_LENSSPECIFICATION     = 0xa432,
   MDT_LENSMAKE              = 0xa433,
   MDT_LENSMODEL             = 0xa434,
   MDT_LENSSERIALNUMBER      = 0xa435,

   // GPS tags (Exif level 3)
   MDT_LEVEL3_BEGIN			 = 0xb000,
   MDT_GPSVERSIONID          = 0xb000,
   MDT_GPSLATITUDEREF        = 0xb001,
   MDT_GPSLATITUDE           = 0xb002,
   MDT_GPSLONGITUDEREF       = 0xb003,
   MDT_GPSLONGITUDE          = 0xb004,
   MDT_GPSALTITUDEREF        = 0xb005,
   MDT_GPSALTITUDE           = 0xb006,
   MDT_GPSTIMESTAMP          = 0xb007,
   MDT_GPSSATELLITES         = 0xb008,
   MDT_GPSSTATUS             = 0xb009,
   MDT_GPSMEASUREMODE        = 0xb00a,
   MDT_GPSDOP                = 0xb00b,
   MDT_GPSSPEEDREF           = 0xb00c,
   MDT_GPSSPEED              = 0xb00d,
   MDT_GPSTRACKREF           = 0xb00e,
   MDT_GPSTRACK              = 0xb00f,
   MDT_GPSIMGDIRECTIONREF    = 0xb010,
   MDT_GPSIMGDIRECTION       = 0xb011,
   MDT_GPSMAPDATUM           = 0xb012,
   MDT_GPSDESTLATITUDEREF    = 0xb013,
   MDT_GPSDESTLATITUDE       = 0xb014,
   MDT_GPSDESTLONGITUDEREF   = 0xb015,
   MDT_GPSDESTLONGITUDE      = 0xb016, 
   MDT_GPSDESTBEARINGREF     = 0xb017,
   MDT_GPSDESTBEARING        = 0xb018,
   MDT_GPSDESTDISTANCEREF    = 0xb019,
   MDT_GPSDESTDISTANCE       = 0xb01a,

   // new GPS tags for Exif 2.2
   MDT_GPSPROCESSINGMETHOD   = 0xb01b,
   MDT_GPSAREAINFORMATION    = 0xb01c,
   MDT_GPSDATESTAMP          = 0xb01d,
   MDT_GPSDIFFERENTIAL       = 0xb01e,
    // new GPS tags for Exif 2.3
   MDT_GPSHPOSITIONINGERROR  = 0xb01f,

   // tags used by IFD1 (thumbnail image)
   MDT_COMPRESSION           = 0x0103,
   MDT_JPEGIFOFFSET          = 0x0201,
   MDT_JPEGIFBYTECOUNT       = 0x0202,

   //Mandatory tags used by Interoperability IFD
   MDT_LEVEL4_BEGIN			   = 0xc000,
   MDT_INTEROPERABILITYINDEX   = 0xc001, //exif 2.21
   MDT_INTEROPERABILITYVERSION = 0xc002, //exif 2.21
};
#endif

#endif