//-------------------------------------------------------------------
// MetadataUtil.cpp
//-------------------------------------------------------------------
// ACD SYSTEMS INTERNATIONAL INC.
// Copyright © 1994 - 2020 ACD Systems International Inc.
// All Rights Reserved
//
// NOTICE: ACD Systems International Inc. ("ACD") permits you to use,
// modify, and distribute this file in accordance with the terms of
// the ACDSee SDK license agreement accompanying it. If you have 
// received this file from a source other than ACD, then your use,
// modification, or distribution of it requires the prior written 
// permission of ACD.
//-------------------------------------------------------------------

/********************************************************************

   Useful utility functions for dealing with metadata.

********************************************************************/

#include "MetadataUtil.h"
#include "Metadata.h"

template<class PTR> void TableDeleteFunc_None(PTR pItem)  {};

///////////////////////////////////////////////////////////////////////////////
// CMetadataList

CMetadataList::CMetadataList() : m_tItems(32, TableDeleteFunc_None)
{
   m_nRefCnt = 1;
}

CMetadataList::~CMetadataList()
{
}

int CMetadataList::GetMetadataCount()
{
   return m_tItems.Count();
}

MD_MetadataItem* CMetadataList::GetMetadataItem(int idx)
{
   MD_MetadataItem* pItem = m_tItems[idx];
   pItem->AddRef();
   return pItem;
}

int CMetadataList::AddMetadataItem(MD_MetadataItem* pItem)
{
   pItem->AddRef();
   RemoveItem(pItem->GetMetadataType());
   m_tItems.Add(pItem);
   return m_tItems.Count() - 1;
}

void CMetadataList::RemoveItem(int idx)
{
   m_tItems[idx]->Release();
   m_tItems.Delete(idx);
}

void CMetadataList::RemoveItem(GUID guidType)
{
   for (int i = m_tItems.Count() - 1; i >= 0 ; i--)
   {
      MD_MetadataItem* pItem = m_tItems[i];
      if (pItem->GetMetadataType() == guidType)
         RemoveItem(i);
   }
}

void CMetadataList::Release()
{
   m_nRefCnt--;
   if (m_nRefCnt == 0)
   {
      for (int i = 0; i < m_tItems.Count(); i++)
        m_tItems[i]->Release();

      delete this;
   }
}

int CMetadataList::AddRef()
{
   return ++m_nRefCnt;
}

///////////////////////////////////////////////////////////////////////////////
// CMetadataItem

CMetadataItem::CMetadataItem()
{
   m_nRefCnt = 1;
}

GUID CMetadataItem::GetMetadataType()
{
   return m_guid;
}

void CMetadataItem::Release()
{
   m_nRefCnt--;
   if (m_nRefCnt == 0)
      delete this;
}

int CMetadataItem::AddRef()
{
   return ++m_nRefCnt;
}

///////////////////////////////////////////////////////////////////////////////
// CExifMetadataItem

CExifMetadataItem::CExifMetadataItem(MD_Item* pItem)
{
   m_pItem = pItem;
   m_guid  = METADATA_EXIF;
}

void CExifMetadataItem::Util_FreeMemory(MD_Item* pItem)
{
   while (pItem)
   {
      MD_Item* pNext = pItem->pNext;
      if (pItem->nTypeID > 0 && pItem->nTypeID <= MD_NumTypes)
      {
         UINT cbLen = MD_TypeSizeTable[pItem->nTypeID] * pItem->nCount;
         if (cbLen > 4) delete[] pItem->pData;
      }
      delete pItem;
      pItem = pNext;
   }
}
CExifMetadataItem::~CExifMetadataItem()
{
   CExifMetadataItem::Util_FreeMemory(m_pItem);
   m_pItem = NULL;
}

void* CExifMetadataItem::GetMetadataData()
{
   return (void*)m_pItem;
}

MD_Item* CExifMetadataItem::GetExifMetadata()
{
   return m_pItem;
}

///////////////////////////////////////////////////////////////////////////////
// CExifMetadataItem

CMediaMetadataItem::CMediaMetadataItem(Media_Item* pItem)
{
   m_pItem = pItem;
   m_guid  = METADATA_MULTIMEDIA;
}

CMediaMetadataItem::~CMediaMetadataItem()
{
   while (m_pItem)
   {
      Media_Item* pNext = m_pItem->pNext;
      if (m_pItem->nTypeID > 0 && m_pItem->nTypeID <= Media_NumTypes)
      {
         UINT cbLen = Media_TypeSizeTable[m_pItem->nTypeID] * m_pItem->nCount;
         if (cbLen > 4) delete[] (BYTE*)m_pItem->pData;
      }
      delete m_pItem;
      m_pItem = pNext;
   }   
   m_pItem = NULL;
}

void* CMediaMetadataItem::GetMetadataData()
{
   return (void*)m_pItem;
}

Media_Item* CMediaMetadataItem::GetMediaMetadata()
{
   return m_pItem;
}

///////////////////////////////////////////////////////////////////////////////
// CIptcMetadataItem

CIptcMetadataItem::CIptcMetadataItem(IPTC_Item* pItem)
{
   m_pItem = pItem;
   m_guid  = METADATA_IPTC;
}

void* CIptcMetadataItem::GetMetadataData()
{
   return (void*)m_pItem;
}

IPTC_Item* CIptcMetadataItem::GetIptcMetadata()
{
   return m_pItem;
}

void CIptcMetadataItem::Util_FreeMemory(IPTC_Item* pItem)
{
   while (pItem)
   {
      IPTC_Item* pNext = pItem->pNext;

      if (pItem->nCount == 1)
      {
         delete[] pItem->pszData;
         pItem->pszData = NULL;
      }
      else
      {
         for (int i = 0; i < pItem->nCount; i++)
         {
            delete[] pItem->ppszData[i];
         }

         delete[] pItem->ppszData;
         pItem->ppszData = NULL;
      }
      delete pItem;

      pItem = pNext;
   }   

}
CIptcMetadataItem::~CIptcMetadataItem()
{
   CIptcMetadataItem::Util_FreeMemory(m_pItem);
   m_pItem = NULL;
}

///////////////////////////////////////////////////////////////////////////////
// CAcdMetadataItem

CAcdMetadataItem::CAcdMetadataItem(ACD_Item* pItem)
{
   m_pItem = pItem;
   m_guid  = METADATA_ACD;
}

void* CAcdMetadataItem::GetMetadataData()
{
   return (void*)m_pItem;
}

ACD_Item* CAcdMetadataItem::GetAcdMetadata()
{
   return m_pItem;
}

CAcdMetadataItem::~CAcdMetadataItem()
{
   while (m_pItem)
   {
      ACD_Item* pNext = m_pItem->pNext;

      if (m_pItem->nCount == 1)
      {
         delete[] m_pItem->pszData;      
      }
      else
      {
         for (int i = 0; i < m_pItem->nCount; i++)
            delete[] m_pItem->ppszData[i];
         delete[] m_pItem->ppszData;
      }
      delete m_pItem;

      m_pItem = pNext;
   }   
   m_pItem = NULL;
}

///////////////////////////////////////////////////////////////////////////////
// CBlobMetadataItem

CBlobMetadataItem::CBlobMetadataItem(GUID guid, BYTE* pBlob, DWORD dwSize)
{
   m_guid   = guid;
   m_pBlob  = new BYTE[sizeof(DWORD) + dwSize];

   *((DWORD*)m_pBlob) = dwSize;

   memcpy(m_pBlob + sizeof(DWORD), pBlob, dwSize);
}

CBlobMetadataItem::CBlobMetadataItem()
{
}

CBlobMetadataItem::~CBlobMetadataItem()
{
   if (m_pBlob)
   {
      delete[] m_pBlob;
      m_pBlob = NULL;
   }
}

void* CBlobMetadataItem::GetMetadataData()
{
   return (void*)m_pBlob;
}

///////////////////////////////////////////////////////////////////////////////
// CFileMetadataItem

CFileMetadataItem::CFileMetadataItem(GUID guid, LPCTSTR pszFile)
{
   m_pBlob  = NULL;
   m_guid   = guid;

   HANDLE hFile = CreateFile(pszFile, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
   if (hFile != INVALID_HANDLE_VALUE)
   {
      DWORD dwSize = GetFileSize(hFile, NULL);
      m_pBlob = new BYTE[sizeof(DWORD) + dwSize];

      DWORD dwRead = 0;
      ReadFile(hFile, m_pBlob + sizeof(DWORD), dwSize, &dwRead, NULL);      
      *((DWORD*)m_pBlob) = dwRead;

      CloseHandle(hFile);
   }
}

CFileMetadataItem::~CFileMetadataItem()
{
}

///////////////////////////////////////////////////////////////////////////////
// CStringMetadataItem

CStringMetadataItem::CStringMetadataItem(GUID guidType, const char* pszString) : CBlobMetadataItem(guidType, (BYTE*)pszString, (DWORD)strlen(pszString) + 1)
{
}

CStringMetadataItem::~CStringMetadataItem()
{
}
char * CStringMetadataItem::GetString()
{
      return (char *)((char*)GetMetadataData() + sizeof(DWORD));
}

CDWORDMetadataItem::CDWORDMetadataItem(GUID guidType, DWORD data) : CBlobMetadataItem(guidType, (BYTE*)&data, sizeof(DWORD))
{
}

CDWORDMetadataItem::~CDWORDMetadataItem()
{
}

DWORD CDWORDMetadataItem::GetDWORD()
{
   DWORD dwRet;
   dwRet = *(DWORD*)(m_pBlob +sizeof(DWORD));
   return dwRet;
}

///////////////////////////////////////////////////////////////////////////////
// CImageResolutionMetadata

ImageResolution::ImageResolution()
{
   m_nSize        = sizeof(ImageResolution);
   m_nXResolution = 0;
   m_nYResolution = 0;
}

CImageResolutionMetadata::CImageResolutionMetadata(GUID guidType, ImageResolution* pImageResolution) 
: CBlobMetadataItem(guidType, (BYTE*)pImageResolution, pImageResolution->m_nSize)
{
}

CImageResolutionMetadata::~CImageResolutionMetadata()
{
}

/*********************
 * Utility Functions *
 *********************/

MD_Item* GetExifMetadataFromList(MD_MetadataList* pList)
{
   if (pList == NULL) return NULL;
   MD_Item* pRet = NULL;
   for (int i = 0; i < pList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == METADATA_EXIF)
         pRet = (MD_Item*)pItem->GetMetadataData();
      pItem->Release();
   }
   return pRet;
}

IPTC_Item* GetIptcMetadataFromList(MD_MetadataList* pList)
{
   if (pList == NULL) return NULL;
   IPTC_Item* pRet = NULL;
   for (int i = 0; i < pList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == METADATA_IPTC)
         pRet = (IPTC_Item*)pItem->GetMetadataData();
      pItem->Release();
   }
   return pRet;
}

DWORD GetIptcDigestFromList(MD_MetadataList* pList)
{
   if (pList == NULL) return NULL;
   DWORD dwRet = IPTC_DIGEST_MISSING;
   for (int i = 0; i < pList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == METADATA_IPTCDIGEST)
         dwRet = ((CDWORDMetadataItem*)pItem)->GetDWORD();
      pItem->Release();
   }
   return dwRet;
}

ACD_Item* GetAcdMetadataFromList(MD_MetadataList* pList)
{
   if (pList == NULL) return NULL;
   ACD_Item* pRet = NULL;
   for (int i = 0; i < pList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == METADATA_ACD)
         pRet = (ACD_Item*)pItem->GetMetadataData();
      pItem->Release();
   }
   return pRet;
}

Media_Item* GetMediaMetadataFromList(MD_MetadataList* pList)
{
   if (pList == NULL) return NULL;
   Media_Item* pRet = NULL;
   for (int i = 0; i < pList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == METADATA_MULTIMEDIA)
         pRet = (Media_Item*)pItem->GetMetadataData();
      pItem->Release();
   }
   return pRet;
}

BYTE* GetColorProfileFromList(MD_MetadataList* pList, DWORD* pdwSize)
{
   if (pList == NULL) return NULL;
   BYTE* pRet = NULL;
   for (int i = 0; i < pList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == METADATA_COLORPROFILE)
      {
         BYTE* pData = (BYTE*)pItem->GetMetadataData();
         if (pdwSize)
            *pdwSize = *((DWORD*)pData);

         pRet = pData + sizeof(DWORD);
      }
      pItem->Release();
   }
   return pRet;
}

BYTE* GetAudioFromList(MD_MetadataList* pList, DWORD* pdwSize)
{
   if (pList == NULL) return NULL;
   BYTE* pRet = NULL;
   for (int i = 0; i < pList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == METADATA_AUDIO)
      {
         BYTE* pData = (BYTE*)pItem->GetMetadataData();
         if (pData)
         {
            if (pdwSize)
               *pdwSize = *((DWORD*)pData);

            pRet = pData + sizeof(DWORD);
         }
      }
      pItem->Release();
   }
   return pRet;
}

char* GetCommentFromList(MD_MetadataList* pList, DWORD* pdwSize)
{
   if (pList == NULL) return NULL;
   char* pRet = NULL;
   for (int i = 0; i < pList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == METADATA_COMMENT)
      {
         BYTE* pData = (BYTE*)pItem->GetMetadataData();
         if (pdwSize)
            *pdwSize = *((DWORD*)pData);

         pRet = (char*) pData + sizeof(DWORD);
      }
      pItem->Release();
   }
   return pRet;
}

char* GetACDDefinedMetadataFromList(MD_MetadataList* pList, DWORD* pdwSize)
{
   if (pList == NULL) return NULL;
   char* pRet = NULL;
   for (int i = 0; i < pList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == METADATA_ACDDEFINEDXML)
      {
         BYTE* pData = (BYTE*)pItem->GetMetadataData();
         if (pdwSize)
            *pdwSize = *((DWORD*)pData);

         pRet = (char*) pData + sizeof(DWORD);
      }
      pItem->Release();
   }
   return pRet;
}

char* GetXmpMetadataFromList(MD_MetadataList* pList, DWORD* pdwSize)
{
   if (pList == NULL) return NULL;
   char* pRet = NULL;
   for (int i = 0; i < pList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == METADATA_XMP)
      {
         BYTE* pData = (BYTE*)pItem->GetMetadataData();
         if (pdwSize)
            *pdwSize = *((DWORD*)pData);

         pRet = (char*) pData + sizeof(DWORD);
      }
      pItem->Release();
   }
   return pRet;
}

ImageResolution* GetImageResolutionFromList(MD_MetadataList* pList, DWORD* pdwSize)
{
   if (pList == NULL) return NULL;
   ImageResolution* pRet = NULL;
   for (int i = 0; i < pList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == METADATA_IMAGERESOLUTION)
      {
         BYTE* pData = (BYTE*)pItem->GetMetadataData();

         DWORD dwSize = *((DWORD*)pData);                  
         if (dwSize >= sizeof(ImageResolution))
         {
            pRet = (ImageResolution*) (pData + sizeof(DWORD));

            if (pdwSize)
            {
               *pdwSize = dwSize;
            }
         }
      }
      pItem->Release();
   }
   return pRet;
}

BOOL IsMetadataInList(MD_MetadataList* pMetadataList, GUID guidType)
{
   if (pMetadataList == NULL) return FALSE;

   BOOL bRes = FALSE;
   for (int i = 0; i < pMetadataList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pMetadataList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == guidType)
         bRes = TRUE;
      pItem->Release();
   }
   return bRes;
}

MD_MetadataItem* GetMetadataInList(MD_MetadataList* pMetadataList, GUID guidType)
{
   if (pMetadataList == NULL) return FALSE;

   for (int i = 0; i < pMetadataList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pMetadataList->GetMetadataItem(i);
      if (pItem->GetMetadataType() == guidType)
      {
         return pItem;
      }
      else
      {
         pItem->Release();
      }
   }
   return NULL;
}

void MergeMetadataLists(MD_MetadataList* pDstList, MD_MetadataList* pSrcList)
{
   if (pDstList == NULL || pSrcList == NULL) return;

   for (int i = 0; i < pSrcList->GetMetadataCount(); i++)
   {
      MD_MetadataItem* pItem = pSrcList->GetMetadataItem(i);
      if (!IsMetadataInList(pDstList, pItem->GetMetadataType()))
      {
         pDstList->AddMetadataItem(pItem);
      }
      pItem->Release();
   }
}
