//-------------------------------------------------------------------
// MetadataUtil.h
//-------------------------------------------------------------------
// ACD SYSTEMS INTERNATIONAL INC.
// Copyright © 1994 - 2020 ACD Systems International Inc.
// All Rights Reserved
//
// NOTICE: ACD Systems International Inc. ("ACD") permits you to use,
// modify, and distribute this file in accordance with the terms of
// the ACDSee SDK license agreement accompanying it. If you have 
// received this file from a source other than ACD, then your use,
// modification, or distribution of it requires the prior written 
// permission of ACD.
//-------------------------------------------------------------------

/********************************************************************

   Useful utility functions for dealing with metadata.

********************************************************************/
#pragma once

#include "../ID_PlugIn.h"
#include "Templates\Table.h"

#define XMP_NS_ACDSEE "http://ns.acdsee.com/iptc/1.0/"
#define XMP_NS_ACDSEE_SHORT "acdsee"

#define XMP_NS_ACDSEEMP "http://ns.acdsee.com/MetadataPreset/1.0/"
#define XMP_NS_ACDSEEMP_SHORT "acdseemp"

#define XMP_NS_ACDSEE_RS "http://ns.acdsee.com/regions/"
#define XMP_NS_ACDSEE_RS_SHORT "acdsee-rs"

#define XMP_NS_ACDSEE_STAREA "http://ns.acdsee.com/sType/Area#"
#define XMP_NS_ACDSEE_STRAREA_SHORT "acdsee-stArea"

#define XMP_NS_ACDSEE_STDIM "http://ns.acdsee.com/sType/Dimensions#"
#define XMP_NS_ACDSEE_STDIM_SHORT "acdsee-stDim"

/*****************
 * CMetadataList *
 *****************/
class CMetadataList : public MD_MetadataList
{
public:
   CMetadataList();
   virtual int GetMetadataCount();
   virtual MD_MetadataItem* GetMetadataItem(int idx);
   virtual int AddMetadataItem(MD_MetadataItem* pItem);
   virtual void RemoveItem(int idx);
   virtual void RemoveItem(GUID guidType);
   virtual void Release();
   virtual int AddRef();
protected:
   virtual ~CMetadataList();
   Table<MD_MetadataItem> m_tItems;
   int m_nRefCnt;
};

/*****************
 * CMetadataItem *
 *****************/
class CMetadataItem : public MD_MetadataItem
{
public:
   virtual ~CMetadataItem() = default;

   virtual GUID GetMetadataType();
   virtual void Release();
   virtual int AddRef();
protected:
   CMetadataItem();
   GUID m_guid;
   int m_nRefCnt;
};

/*********************
 * CExifMetadataItem *
 *********************/
class CExifMetadataItem : public CMetadataItem
{
public:
   CExifMetadataItem(MD_Item* pItem);
   virtual void* GetMetadataData();
   MD_Item* GetExifMetadata();
   static void Util_FreeMemory(MD_Item* pItem);
protected:
   virtual ~CExifMetadataItem();
   MD_Item* m_pItem;
};

/*********************
 * CMediaMetadataItem *
 *********************/
class CMediaMetadataItem : public CMetadataItem
{
public:
   CMediaMetadataItem(Media_Item* pItem);
   virtual void* GetMetadataData();
   Media_Item* GetMediaMetadata();
protected:
   virtual ~CMediaMetadataItem();
   Media_Item* m_pItem;
};

/*********************
 * CIptcMetadataItem *
 *********************/
class CIptcMetadataItem : public CMetadataItem
{
public:
   CIptcMetadataItem(IPTC_Item* pItem);
   virtual void* GetMetadataData();
   IPTC_Item* GetIptcMetadata();
   static void Util_FreeMemory(IPTC_Item* pItem);
protected:
   virtual ~CIptcMetadataItem();
   IPTC_Item* m_pItem;
};

/*********************
 * CAcdMetadataItem *
 *********************/
class CAcdMetadataItem : public CMetadataItem
{
public:
   CAcdMetadataItem(ACD_Item* pItem);
   virtual void* GetMetadataData();
   ACD_Item* GetAcdMetadata();
protected:
   virtual ~CAcdMetadataItem();
   ACD_Item* m_pItem;
};

/*********************
 * CBlobMetadataItem *
 *********************/
class CBlobMetadataItem : public CMetadataItem
{
public:
   CBlobMetadataItem(GUID guid, BYTE* pBlob, DWORD dwSize);
   virtual void* GetMetadataData();
protected:
   CBlobMetadataItem();
   virtual ~CBlobMetadataItem();

   BYTE* m_pBlob;
};

/*********************
 * CFileMetadataItem *
 *********************/
class CFileMetadataItem : public CBlobMetadataItem
{
public:
   CFileMetadataItem(GUID guid, LPCTSTR pszFile);
protected:
   virtual ~CFileMetadataItem();
};

/***********************
 * CStringMetadataItem *
 ***********************/
class CStringMetadataItem : public CBlobMetadataItem
{
public:
   CStringMetadataItem(GUID guidType, const char* pszString);
   char* GetString();
protected:
   virtual ~CStringMetadataItem();
};

/***********************
 * CDWORDMetadataItem *
 ***********************/
class CDWORDMetadataItem : public CBlobMetadataItem
{
public:
   CDWORDMetadataItem(GUID guidType, DWORD);
   DWORD GetDWORD();
protected:
   virtual ~CDWORDMetadataItem();
};

struct ImageResolution
{
   ImageResolution();

   UINT m_nSize;
   UINT m_nXResolution;
   UINT m_nYResolution;
};

class CImageResolutionMetadata : public CBlobMetadataItem
{
public:
   CImageResolutionMetadata(GUID guidType, ImageResolution* pImageResolution);
protected:
   virtual ~CImageResolutionMetadata();
};

/*********************
 * Utility Functions *
 *********************/

MD_Item* GetExifMetadataFromList(MD_MetadataList*);
IPTC_Item* GetIptcMetadataFromList(MD_MetadataList* pList);
DWORD GetIptcDigestFromList(MD_MetadataList* pList);
ACD_Item* GetAcdMetadataFromList(MD_MetadataList* pList);
Media_Item* GetMediaMetadataFromList(MD_MetadataList* pList);
BYTE* GetColorProfileFromList(MD_MetadataList*, DWORD* pdwSize = NULL);
BYTE* GetAudioFromList(MD_MetadataList*, DWORD* pdwSize = NULL);
char* GetCommentFromList(MD_MetadataList* pMetadataList, DWORD* pdwSize = NULL);
char* GetXmpMetadataFromList(MD_MetadataList* pMetadataList, DWORD* pdwSize = NULL);
ImageResolution* GetImageResolutionFromList(MD_MetadataList* pList, DWORD* pdwSize = NULL);

BOOL IsMetadataInList(MD_MetadataList* pMetadataList, GUID guidType);
MD_MetadataItem* GetMetadataInList(MD_MetadataList* pMetadataList, GUID guidType);
void MergeMetadataLists(MD_MetadataList* pDstList, MD_MetadataList* pSrcList);
char* GetACDDefinedMetadataFromList(MD_MetadataList* pList, DWORD* pdwSize = NULL);
