//-------------------------------------------------------------------
// Table.h
//-------------------------------------------------------------------
// ACD SYSTEMS INTERNATIONAL INC.
// Copyright © 1994 - 2020 ACD Systems International Inc.
// All Rights Reserved
//
// NOTICE: ACD Systems International Inc. ("ACD") permits you to use,
// modify, and distribute this file in accordance with the terms of
// the ACDSee SDK license agreement accompanying it. If you have 
// received this file from a source other than ACD, then your use,
// modification, or distribution of it requires the prior written 
// permission of ACD.
//-------------------------------------------------------------------

#pragma once

#include <stdlib.h>

/*********
 * Table *
 *********
 * Class template that can manage any type of record (dynamic array representation).
 * Record references are used.  Thus items interted must be dynamically allocated.
 * A table owns its records if OwnsElements(1) has been called.
 * When a table is destroyed, the records it owns are destroyed with it.
 * Likewise, when an item is deleted from a table that owns its elements, that item is destroyed.
 * The default table size is 32 records.  The size will grow dynamically as records are added.
 * A custom delete handler can be specified in the constructor.  The default handler will
 * delete the items in the table using the delete operator in the C++ runtime.
 */

// Deletion handlers
template<class PTR> void TableDeleteFunc_Delete(PTR pItem)           { if (pItem!=NULL) delete pItem; pItem = NULL;};
template<class PTR> void TableDeleteFunc_DeleteArray(PTR pItem)      { delete[] pItem;    };
template<class PTR> void TableDeleteFunc_Free(PTR pItem)             { free(pItem);       };
template<class PTR> void TableDeleteFunc_Release(PTR pItem)          { pItem->Release();  };
template<class PTR> void TableDeleteFunc_SmartPtrRelease(PTR pItem)  { pItem = NULL;      };

template<class R, class PTR = R*> 
class Table
{
   public:
      // Constructs a Table pre-allocated to hold nSize elements
      Table(int nSize=32, void (*pfDeleteFunc)(PTR pItem) = TableDeleteFunc_Delete);

      // Constructs a Table with a specified delete handler
      Table(void (*pfDeleteFunc)(PTR pItem));

      // Constucts a Table as a copy of another Table object
      Table(const Table<R, PTR>&);

      // Destructor
      virtual ~Table();

      // Sets Table to the contents of another Table
      Table<R, PTR>& operator=(const Table<R, PTR>& ct);

      // Adds an item to the table
      void Add(PTR);

      // Adds all items from ct to the table
      void Add(const Table<R, PTR>& ct);

      // Removes an item and destroys it if OwnsElements() is true.
      void Delete(int i);
      void Delete(const PTR);

      // Removes an item without destroying it
      void Remove(int i);

      // Changes the item at position i to pItem
      void Change(int i, const PTR pItem);

      // Inserts an item at position iAt
      void Insert(int iAt, const PTR pItem);

      // Moves an item from iFrom into position iTo (moving all items following)
      void MoveItem(int iFrom, int iTo);

      // Returns the index of the given item within the Table
      int IndexOf(const PTR pItem) const;

      // Sets the ownership property of the Table
      void OwnsElements(int i)      { owner = i; };

      // Returns the ownership property of the Table
      int OwnsElements()     const { return owner; };

      // Returns the item at position i
      PTR ItemAt(int i)      const { return ((i>=count || i<0) ? NULL : items[i]); };
      PTR operator[](int i)  const { return ((i>=count || i<0) ? NULL : items[i]); };

      // Returns the number of items in the Table
      int Count() const { return count; };

      // Deletes all items in the table
      void RemoveAll();

      // Iterators:
      // Resets the iterator to the first element and returns it
      PTR First() { return ItemAt(cur=0); };

      // Returns the next element
      PTR Next() { return cur<count ? ItemAt(++cur) : NULL; };

      // Returns the current element
      PTR Cur() const { return ItemAt(cur); };

      int GetGrowthDelta() const { return delta; };
      void SetGrowthDelta(const int iNewDelta) { iNewDelta > 0 ? delta = iNewDelta : delta = DefaultGrowthDelta; };

      static const int DefaultGrowthDelta = 64;

   protected:
      PTR* items;    // Array of items
      int count;    // Number of items in Table
      int max;      // Size of array
      int owner;    // Flag: ownership property of Table
      int delta;    // Number of items by which to grow the array when count exceeds max
      int cur;      // Index of current iterator item
      void (*pfDeleteFunc)(PTR pItem);
};


/////////////////////////////////////////////////////////////////////
// Helper functions.

// Makes a deep copy of a Table template object.
// Use this version for item classes that have the standard copy-ctor: T(const T& rhs).
template<typename T, typename PTR>
void CopyTableDeep(const Table<T, PTR>& src, Table<T, PTR>& dst);

// Makes a deep copy of a Table template object.
// Use this version for item classes that have a copy-ctor that
// takes a pointer to the source object, e.g. see CCategory or CCalendarDateRange: T(T* pRhs).
template<typename T, typename PTR> 
void CopyTableDeep_PointerCopyCtor(const Table<T, PTR>& src, Table<T, PTR>& dst);

//////////////////////////////////////////////////////////////////////


/****************
 * Table::Table *
 ****************/
template<class R, class PTR> 
Table<R, PTR>::Table(void (*_pfDeleteFunc)(PTR pItem)) 
{
   int size = 32;

   items        = new PTR[size];
   count        = 0;
   max          = size-1;
   owner        = 0;
   delta        = DefaultGrowthDelta;
   pfDeleteFunc = _pfDeleteFunc;
}

template<class R, class PTR> 
Table<R, PTR>::Table(int size, void (*_pfDeleteFunc)(PTR pItem)) 
{
   items        = new PTR[size];
   count        = 0;
   max          = size-1;
   owner        = 0;
   delta        = DefaultGrowthDelta;
   pfDeleteFunc = _pfDeleteFunc;
}

template<class R, class PTR> 
Table<R, PTR>::Table(const Table<R, PTR>& ct) 
{
   count = ct.count;
   max   = ct.max;
   pfDeleteFunc = ct.pfDeleteFunc;
   owner = 0;
   delta = DefaultGrowthDelta;
   items = new PTR[max+1];

   for (int i = 0; i < ct.count; i++)
   {
      items[i] = ct.items[i];
   }
}

/*****************
 * Table::~Table *
 *****************
 * We need to go through and destroy all the records if we own them.
 */
template<class R, class PTR>
Table<R, PTR>::~Table()
{
   if (owner)
      for (int i=0 ; i<count ; i++)
         pfDeleteFunc(ItemAt(i));
   delete[] items;
}


/********************
 * Table::operator= *
 ********************/
template<class R, class PTR>
Table<R, PTR>& Table<R, PTR>::operator=(const Table<R, PTR>& ct)
{
   if (owner)
      for (int i=0 ; i<count ; i++)
         pfDeleteFunc(ItemAt(i));
   delete[] items;
   count = ct.count;
   max   = ct.max;
   owner = 0;
   delta = DefaultGrowthDelta;
   items = new PTR[max+1];
   
   for (int i = 0; i < ct.count; i++)
   {
      items[i] = ct.items[i];
   }
   
   return *this;
}

/**************
 * Table::Add *
 **************
 * Adds the element to the table
 */
template<class R, class PTR>
void Table<R, PTR>::Add(PTR r)
{
   if (!r)
   {
      return;
   }

   if (count == max+1)
   {
      PTR* newItems = new PTR[max+delta+1];
      
      for (int i = 0; i < max+1; i++)
      {
         newItems[i] = items[i];
      }

      delete[] items;
      items = newItems;
      max += delta;
   }

   items[count] = r;
   count++;
}

// Adds all items from ct to the table
template<class R, class PTR>
void Table<R, PTR>::Add(const Table<R, PTR>& ct)
{
   if (ct.count == 0)
   {
      return;
   }

   if (count + ct.count >= max + 1)
   {
      max += ct.count+delta;
      PTR* newItems = new PTR[max+1];

      for (int i = 0; i < count; i++)
      {
         newItems[i] = items[i];
      }

      delete[] items;
      items = newItems;
   }

   for (int i = 0; i < ct.count; i++)
   {
      items[count+i] = ct.items[i];
   }
   count += ct.count;
}

/*****************
 * Table::Delete *
 *****************
 * Removes the element from the table.
 * If the table owns the element, it is destroyed.
 */
template<class R, class PTR>
void Table<R, PTR>::Delete(int i)
{
   if (i < 0 || i >= count) return;
   if (owner)
      pfDeleteFunc(items[i]);
   Remove(i);
}

template<class R, class PTR>
void Table<R, PTR>::Delete(const PTR r)
{
   Delete(IndexOf(r));
}

template<class R, class PTR>
void Table<R, PTR>::Remove(int i)
{
   if (i < 0 || i >= count) return;
   for (int j=i ; j<count-1 ; j++)
      items[j] = items[j+1];
   items[count - 1] = NULL;
   count--;
}

/*****************
 * Table::Change *
 *****************/
template<class R, class PTR>
void Table<R, PTR>::Change(int i, const PTR r)
{
   if (i < 0 || i >= count) return;

   if (owner)
       pfDeleteFunc(items[i]);

   items[i] = (PTR)r;
}

/*****************
 * Table::Insert *
 *****************/
template<class R, class PTR>
void Table<R, PTR>::Insert(int iAt, const PTR r)
{
   if (iAt < 0 || iAt > Count()) return;
   Add(r);
   int i=0;
   for (i=Count()-1 ; i>iAt ; i--)
   {
      items[i] = items[i-1];
   }
   items[i] = (PTR)r;
}

template<class PTR>	
int cmp(const PTR* r1, const PTR* r2)
{
   if (**r1 <  **r2)
      return -1;
   if (**r1 == **r2)
      return 0;
   return 1;
}

/*******************
 * Table::MoveItem *
 *******************/
template<class R, class PTR>
void Table<R, PTR>::MoveItem(int iFrom, int iTo)
{
    if (iFrom < 0 || iFrom >= count) return;
    if (iTo   < 0 || iTo   >= count) return;
    if (iFrom == iTo) return;

    PTR prFrom = items[iFrom];

    // Remove "From" item and shift everything over
    memmove(items+iFrom, items+iFrom+1, (count-iFrom-1)*sizeof(PTR));

    // Shift everything over to insert "To" item and 
    memmove(items+iTo+1, items+iTo, (count-iTo-1)*sizeof(PTR));
    items[iTo] = prFrom;
}

/******************
 * Table::IndexOf *
 ******************/
template<class R, class PTR>
int Table<R, PTR>::IndexOf(const PTR r) const
{
   for (int i=0 ; i<count ; i++)
      if (items[i] == r)
         return i;
   return -1;
}
    
/********************
 * Table::RemoveAll *
 ********************/
template<class R, class PTR>
void Table<R, PTR>::RemoveAll()
{
   if (owner)
      for (int i=0 ; i<count ; i++)
         pfDeleteFunc(ItemAt(i));
   delete[] items;
   items = new PTR[32];
   count = 0;
   max   = 31;
   delta = DefaultGrowthDelta;
}     


// Makes a deep copy of a Table template object.
// Use this version for item classes that have the standard copy-ctor: T(const T& rhs).
template<typename T, typename PTR>
inline void CopyTableDeep(const Table<T, PTR>& src, Table<T, PTR>& dst)
{
   dst.RemoveAll();

   for (int itemIdx = 0; itemIdx < src.Count(); ++itemIdx)
   {
      PTR pItemCopy = new T(*src[itemIdx]);
      dst.Add(pItemCopy);
   }

   dst.OwnsElements(1);
}

// Makes a deep copy of a Table template object.
// Use this version for item classes that have a copy-ctor that
// takes a pointer to the source object, e.g. see CCategory or CCalendarDateRange: T(T* pRhs).
template<typename T, typename PTR> 
inline void CopyTableDeep_PointerCopyCtor(const Table<T, PTR>& src, Table<T, PTR>& dst)
{
   dst.RemoveAll();

   for (int itemIdx = 0; itemIdx < src.Count(); ++itemIdx)
   {
      PTR pItemCopy = new T(src[itemIdx]);
      dst.Add(pItemCopy);
   }

   dst.OwnsElements(1);
}
